      subroutine geopar
      use mod_floats  ! HYCOM post-processing floats archive array interface
      use mod_xc      ! HYCOM communication interface
      use mod_za      ! HYCOM array I/O interface
c
      implicit none
c
c --- set up model parameters related to geography
c
      real      hmina,hminb,hmaxa,hmaxb
      integer   l
      character preambl(5)*79,cline*80
c
c --- read basin depth array
c
      write (lp,'(2a)') ' reading bathymetry file from ',
     &                  'regional.depth.[ab]'
      call xcsync(flush_lp)
      open (unit=9,file='regional.depth.b',status='old')
      read (     9,'(a79)')  preambl
      read (     9,'(a)')    cline
      i = index(cline,'=')
      read (cline(i+1:),*)   hminb,hmaxb
      close(unit=9)
      write (lp,'(/(1x,a))') preambl,cline
c
      call zaiopf('regional.depth.a','old', 9)
      call zaiord(depths,ip,.false., hmina,hmaxa, 9)
      call zaiocl(9)
c
      if     (abs(hmina-hminb).gt.abs(hminb)*1.e-4 .or.
     &        abs(hmaxa-hmaxb).gt.abs(hmaxb)*1.e-4     ) then
        write(lp,'(/ a / a,1p3e14.6 / a,1p3e14.6 /)')
     &    'error - .a and .b files not consistent:',
     &    '.a,.b min = ',hmina,hminb,hmina-hminb,
     &    '.a,.b max = ',hmaxa,hmaxb,hmaxa-hmaxb
        stop '(geopar)'
      endif
c
c --- start out with masks as land everywhere
      do j=1,jdm
        do i=1,idm
          ip(i,j)=0
          iu(i,j)=0
          iv(i,j)=0
        enddo
      enddo
c
c --- mass points are defined where water depth is greater than zero
c --- but for GLBa0.08 there are large numbers over land to be masked later.
      do j=1,jdm
        do i=1,idm
          if (depths(i,j).gt.0.) then
            ip(i,j)=1
          endif
        enddo
      enddo
c
c --- u,v points are located halfway between any 2 adjoining mass points
      do j=1,jdm
        do i=1,idm
          if (ip(max(1,i-1),j).gt.0.and.ip(i,j).gt.0) then
            iu(i,j)=1
          endif
          if (ip(i,max(1,j-1)).gt.0.and.ip(i,j).gt.0) then
            iv(i,j)=1
          endif
        enddo
      enddo
c
c --- read grid location,spacing,coriolis arrays
c
      write (lp,'(2a)') ' reading grid file from ',
     &                  'regional.grid.[ab]'
      call xcsync(flush_lp)
      open (unit=9,file='regional.grid.b',status='old')
      read (9,*) i
      read (9,*) j
      if     (i.ne.idm .or. j.ne.jdm) then
        write(lp,'(/ a /)')
     &    'error - wrong array size in grid file'
        call xcstop('(geopar)')
               stop '(geopar)'
      endif
      read (9,'(a)') cline
      write (lp,'(a)') cline(1:len_trim(cline))
c
      call zaiopf('regional.grid.a','old', 9)
c
      do k= 1,12
        read (9,'(a)') cline
        i = index(cline,'=')
        read (cline(i+1:),*) hminb,hmaxb
        write (lp,'(a)') cline(1:len_trim(cline))
        call xcsync(flush_lp)
c
        if     (k.eq.1) then
          call zaiord(plon, ip,.true., hmina,hmaxa, 9)
        elseif (k.eq.2) then
          call zaiord(plat, ip,.true., hmina,hmaxa, 9)
          do i= 1,2
            read (9,'(a)') cline
            call zaiosk(9)
          enddo
        elseif (k.eq.3) then
          call zaiord(ulon, iu,.true., hmina,hmaxa, 9)
        elseif (k.eq.4) then
          call zaiord(ulat, iu,.true., hmina,hmaxa, 9)
        elseif (k.eq.5) then
          call zaiord(vlon, iv,.true., hmina,hmaxa, 9)
        elseif (k.eq.6) then
          call zaiord(vlat, iv,.true., hmina,hmaxa, 9)
          read (9,'(a)') cline
          call zaiosk(9)
        elseif (k.eq.7) then
          call zaiord(scpx, ip,.true., hmina,hmaxa, 9)
        elseif (k.eq.8) then
          call zaiord(scpy, ip,.true., hmina,hmaxa, 9)
          do i= 1,2
            read (9,'(a)') cline
            call zaiosk(9)
          enddo
        elseif (k.eq.9) then
          call zaiord(scux, iu,.true., hmina,hmaxa, 9)
        elseif (k.eq.10) then
          call zaiord(scuy, iu,.true., hmina,hmaxa, 9)
        elseif (k.eq.11) then
          call zaiord(scvx, iv,.true., hmina,hmaxa, 9)
        else
          call zaiord(scvy, iv,.true., hmina,hmaxa, 9)
        endif
c
        if     (abs(hmina-hminb).gt.abs(hminb)*1.e-4 .or.
     &          abs(hmaxa-hmaxb).gt.abs(hmaxb)*1.e-4     ) then
          write(lp,'(/ a / a,1p3e14.6 / a,1p3e14.6 /)')
     &      'error - .a and .b files not consistent:',
     &      '.a,.b min = ',hmina,hminb,hmina-hminb,
     &      '.a,.b max = ',hmaxa,hmaxb,hmaxa-hmaxb
          call xcstop('(geopar)')
                 stop '(geopar)'
        endif
      enddo
c
      do j= 1,jdm
        do i= 1,idm
          scu2(i,j)=scux(i,j)*scuy(i,j)
          scv2(i,j)=scvx(i,j)*scvy(i,j)
        enddo
      enddo
c --- determine if this is a global periodic grid
c
      lperiod = maxval(plon(:,:))-minval(plon(:,:)) .gt. 350.0

c
      call zaiocl(9)
c
!$OMP PARALLEL DO PRIVATE(j,i)
!$OMP&         SCHEDULE(STATIC,jblk)
      do j= 1,jdm
        do i= 1,idm
          if     (depths(i,j).gt.2.0**99) then
            depths(i,j) = 0.0
          endif
        enddo
      enddo
c recompute ip, iu,iv
      do j=1,jdm
         do i=1,idm
            ip(i,j)=0
            iu(i,j)=0
            iv(i,j)=0
            if (depths(i,j).gt.0.)then
            ip(i,j)=1
            endif
         enddo
      enddo
c --- u,v points are located halfway between any 2 adjoining mass points
      do j=1,jdm
        do i=1,idm
          if (ip(max(1,i-1),j).gt.0.and.ip(i,j).gt.0) then
            iu(i,j)=1
          endif
          if (ip(i,max(1,j-1)).gt.0.and.ip(i,j).gt.0) then
            iv(i,j)=1
          endif
        enddo
c adjust for periodic domain
        if(lperiod)then
          iu(1,j)=0
          if (ip(idm,j).gt.0.and.ip(1,j).gt.0)then
            iu(1,j)=1
          endif
        endif
      enddo
c
!$OMP PARALLEL DO PRIVATE(j,i)
!$OMP&         SCHEDULE(STATIC,jblk)
      do j= 1,jdm
        do i= 1,idm
          depthu(i,j)=onem*min(depths(i,j),depths(max(i-1,1),j))
          depthv(i,j)=onem*min(depths(i,j),depths(i,max(j-1,1)))
        enddo
c adjust for periodic domain
        if(lperiod)then
          depthu(1,j)=onem*min(depths(1,j),depths(idm,j))
        endif
      enddo
c print first column
      print *, 'ip,iu,iv, onem*depths,depthu,depthv i=1,3 j=1000'
      print *, ip(1,1000),iu(1,1000),iv(1,1000) 
      print *, (onem*depths(i,1000),depthu(i,1000),depthv(i,1000)
     &         ,i=1,3)
      print *, 'end geopar'
c
      return
      end
