      subroutine geopar_ascii
c
c --- this version initializes a non-hycom domain
d
      use mod_floats  ! HYCOM post-processing floats archive array interface
      use mod_xc      ! HYCOM communication interface
      use mod_za      ! HYCOM array I/O interface
c
      implicit none
c
c --- set up model parameters related to geography
c
      real      hmina,hminb,hmaxa,hmaxb
      integer   l
c
c --- read basin depth array
c
      write (lp,'(2a)') ' generating bathymetry file for ',
     &                  'non-hycom domain{'
c
c --- start out with water everywhere - land mask set during data read
      do j=1,jdm
        do i=1,idm
          depths(i,j)=1.0
          ip(i,j)=0
          iu(i,j)=0
          iv(i,j)=0
        enddo
      enddo
c
c --- read grid location,spacing
c
      write (lp,'(2a)') ' reading grid file from ',
     &                  'regional.grid.[ab]'
c
          call zaiord(plon, ip,.true., hmina,hmaxa, 9)
          call zaiord(plat, ip,.true., hmina,hmaxa, 9)
          call zaiord(scpx, ip,.true., hmina,hmaxa, 9)
          call zaiord(scpy, ip,.true., hmina,hmaxa, 9)
          enddo
c
      do j= 1,jdm
        do i= 1,idm
          scu2(i,j)=scux(i,j)*scuy(i,j)
          scv2(i,j)=scvx(i,j)*scvy(i,j)
        enddo
      enddo
c --- determine if this is a global periodic grid
c
      lperiod = maxval(plon(:,:))-minval(plon(:,:)) .gt. 350.0

c
      call zaiocl(9)
c
!$OMP PARALLEL DO PRIVATE(j,i)
!$OMP&         SCHEDULE(STATIC,jblk)
      do j= 1,jdm
        do i= 1,idm
          if     (depths(i,j).gt.2.0**99) then
            depths(i,j) = 0.0
          endif
        enddo
      enddo
c recompute ip, iu,iv
      do j=1,jdm
         do i=1,idm
            ip(i,j)=0
            iu(i,j)=0
            iv(i,j)=0
            if (depths(i,j).gt.0.)then
            ip(i,j)=1
            endif
         enddo
      enddo
c --- u,v points are located halfway between any 2 adjoining mass points
      do j=1,jdm
        do i=1,idm
          if (ip(max(1,i-1),j).gt.0.and.ip(i,j).gt.0) then
            iu(i,j)=1
          endif
          if (ip(i,max(1,j-1)).gt.0.and.ip(i,j).gt.0) then
            iv(i,j)=1
          endif
        enddo
c adjust for periodic domain
        if(lperiod)then
          iu(1,j)=0
          if (ip(idm,j).gt.0.and.ip(1,j).gt.0)then
            iu(1,j)=1
          endif
        endif
      enddo
c
!$OMP PARALLEL DO PRIVATE(j,i)
!$OMP&         SCHEDULE(STATIC,jblk)
      do j= 1,jdm
        do i= 1,idm
          depthu(i,j)=onem*min(depths(i,j),depths(max(i-1,1),j))
          depthv(i,j)=onem*min(depths(i,j),depths(i,max(j-1,1)))
        enddo
c adjust for periodic domain
        if(lperiod)then
          depthu(1,j)=onem*min(depths(1,j),depths(idm,j))
        endif
      enddo
c print first column
      print *, 'ip,iu,iv, onem*depths,depthu,depthv i=1,3 j=1000'
      print *, ip(1,1000),iu(1,1000),iv(1,1000) 
      print *, (onem*depths(i,1000),depthu(i,1000),depthv(i,1000)
     &         ,i=1,3)
      print *, 'end geopar'
c
      return
      end
