      program floats
      use mod_floats ! HYCOM post-processing synthetic float interface
      use mod_za     ! HYCOM array I/O interface
c
      implicit none
c
c --- post-processing lagrangian floats from a sequence of HYCOM 2.0
c --- archive files.
c
      character*120    flnm1,flnm2
      logical          lexist,lfatal,trcout
      integer          iexpt,yrflag,kdmin
      integer          iinc,it,itr,jinc,jt,nntr
      real             depthi(0:1,0:1,0:99),duk,dukm1,dvk,dvkm1
      double precision time_ai,time_ao,time1,time2
c
      call xcspmd
      call zaiost
c
      lp=6
      onecm=0.01*onem
      tencm=0.10*onem
      iuvgrd=1           ! hycom u,v on staggered c grid
c
      pi = 4.0*atan(1.0)
      radian=pi/180.0
c
c --- 'flnm_fii' = name of input floats input file
c --- 'flnm_fio' = name of output floats input file for next time segment 
c --- 'flnm_fo'  = name of floats output file containing path positions
c --- 'flnm_ai'  = name of first input archive file
c --- 'flnm_ao'  = name of final input archive file
c --- 'iexpt '   = experiment number x10  (000=from archive file)
c --- 'yrflag'   = days in year flag (0=360J16,1=366J16,2=366J01,3=actual)
c --- 'archfq'   = no. of hours between archive input
c --- 'archin'   = no. of hours between time-interpolated archive input
c --- 'idm   '   = longitudinal array size
c --- 'jdm   '   = latitudinal  array size
c --- 'kdm   '   = number of layers
c
      read (*,'(a)') flnm_fii
      write (lp,'(2a)')
     & 'input floats input file:  ',flnm_fii(1:len_trim(flnm_fii))
      call flush(lp)
      read (*,'(a)') flnm_fio
      write (lp,'(2a)')
     & 'output floats input file: ',flnm_fio(1:len_trim(flnm_fio))
      call flush(lp)
      read (*,'(a)') flnm_fo
      write (lp,'(2a)')
     & 'floats output file:       ',flnm_fo(1:len_trim(flnm_fo))
      call flush(lp)
      read (*,'(a)') flnm_ai
      write (lp,'(2a)')
     & 'archive input file:       ',flnm_ai(1:len_trim(flnm_ai))
      call flush(lp)
      read (*,'(a)') flnm_ao
      write (lp,'(2a)')
     & 'archive output file:      ',flnm_ao(1:len_trim(flnm_ao))
      call flush(lp)
c
      call blkini(iexpt, 'iexpt ')
      call blkini(yrflag,'yrflag')
      call blkinr(archfq,
     &           'archfq','("blkinr: ",a6," =",f11.4," hours")')
      call blkinr(archin,
     &           'archin','("blkinr: ",a6," =",f11.4," hours")')
      archfq=archfq/24.0
      archin=archin/24.0
      nintrp=0
      if (archfq.ne.archin) then
        nintrp=nint(archfq/archin)
        if (archfq/archin.ne.float(nintrp)) then
          stop 'archfq must be an integer multiple of archin'
        endif
      endif
      call blkini(ii,    'idm   ')
      call blkini(jj,    'jdm   ')
      call blkini(kdm,   'kdm   ')
      call blkini(kk,    'kk    ')
      if     (ii.ne.idm .or. jj.ne.jdm) then
        write(lp,*)
        write(lp,*) 'error - wrong idm or jdm (should be:',idm,jdm,')'
        write(lp,*)
        call flush(lp)
        stop 
      endif
      call blkinr(thbase,
     &           'thbase','("blkinr: ",a6," =",f11.4," days")')
c
c --- 'fltdbg' = FLOATS: float number to be debugged
c --- 'intpfl' = FLOATS: horiz. interp. (0=2nd order+bilinear; 1=bilinear)
c --- 'dfmonc' = FLOATS: evaporation/termination time scale (days for 50%)
c --- 'iturbv' = FLOATS: add horiz. turb. advection velocity (0=no; 1=yes)
c --- 'tbvar'  = FLOATS: horizontal turbulent velocity variance scale
c --- 'tdecri' = FLOATS: inverse decorrelation time scale
c 
      call blkini(nfl_debug,'fltdbg')
      call blkini(intpfl,'intpfl')
      call blkinr(dfmonc,'dfmonc','(a6," =",f10.4," days")')
      call blkini(iturbv,'iturbv')
      turbvel = iturbv.ge.1
      call blkinr(tbvar ,'tbvar ','(a6," =",f10.4," m**2/s**2")')
      call blkinr(tdecri,'tdecri','(a6," =",f10.4," 1/day")')
c
c --- array allocation
c
      call floats_alloc
c
c --- land masks.
c
      call geopar
c
      open (unit=nt1,file=flnm_fii,form='formatted',
     &      status='old',action='read' )
      open (unit=nt3,file=flnm_fo ,form='formatted',
     &      status='new',action='write')
c
c --- first and last model days from the input archives.
c
      call getdat_time(flnm_ai,time_ai,iexpt)
      call getdat_time(flnm_ao,time_ao,iexpt)
c
      narch = nint((time_ao-time_ai)/archfq) + 1
c
      write(lp,900) flnm_ai,flnm_ao
  900 format('first and last archives files:'//a120/a120)
      write(lp,910) narch,time_ai,time_ao
  910 format(/'number of archives, start and end times:'/i6,
     &       1p,2e16.8)
c
c --- initialize the floats
c
      iar=1
      call getdat_name(flnm_ai,time_ai,yrflag)
      write(lp,'(/a,i4,f10.2,/a)') 'iar,time_ai,flnm_ai = ',
     &                        iar,time_ai,flnm_ai(1:len_trim(flnm_ai))
      call flush(lp)
c
      call getdat(flnm_ai,time_ai,iexpt,yrflag,trcout)
c
      do j=1,jj
        do i=1,ii
          depth1(i,j)=depths(i,j)
        enddo !i
      enddo !j
      call bigrid(depth1)
c
c --- smooth grid-point noise in the velocity fields
      do k=1,kk
c
        do j=1,jj-1
          do i=2,ii-1
              util1(i,j)=(u(i,j,k)+ubaro(i,j))*
     &                    max(onecm,dp(i,j,k)+dp(i-1,j,k))
           enddo !i
        enddo !j
c
        do j=2,jj-1
          do i=1,ii-1
            util2(i,j)=(v(i,j,k)+vbaro(i,j))*
     &                  max(onecm,dp(i,j,k)+dp(i,j-1,k))
          enddo !i
        enddo !j
c
        call usmoo(util1,work)
        call vsmoo(util2,work)
c
        do j=1,jj-1
          do i=2,ii-1
            u(i,j,k)=util1(i,j)/max(onecm,dp(i,j,k)+dp(i-1,j,k))
          enddo !i
        enddo !j
c
        do j=2,jj-1
          do i=1,ii-1
            v(i,j,k)=util2(i,j)/max(onecm,dp(i,j,k)+dp(i,j-1,k))
          enddo !i
        enddo !j
c
      enddo
c
      call floats_init
c     stop 'for debug, stop at end of floats_init'
c
c --- initial call to floats_advect writes out float info. at initial time
c
      call floats_advect
c
      flnm1=flnm_ai
      flnm2=flnm_ai
c
      do iar=2,narch
c
c ---   loop through all input model archives
c
        if (nintrp.gt.1) then
          time1=time_ai+(iar-2)*archfq
          call getdat_name(flnm1,time1,yrflag)
          write(lp,'(a,i4,f10.2,/a)')'iar,time1,flnm1 = ',
     &                                iar,time1,flnm1(1:len_trim(flnm1))
          call flush(lp)
        endif !nintrp.gt.1
        time2=time_ai+(iar-1)*archfq
        call getdat_name(flnm2,time2,yrflag)
        write(lp,'(a,i4,f10.2,/a)') 'iar,time2,flnm2 = ',
     &                               iar,time2,flnm2(1:len_trim(flnm2))
        call flush(lp)
c
        call getdat(flnm2,time2,iexpt,yrflag,trcout)
c
c --- smooth grid-point noise in the velocity fields
        do k=1,kk
c
          do j=1,jj-1
            do i=2,ii-1
                util1(i,j)=(u(i,j,k)+ubaro(i,j))*
     &                      max(onecm,dp(i,j,k)+dp(i-1,j,k))
             enddo !i
          enddo !j
c
          do j=2,jj-1
            do i=1,ii-1
              util2(i,j)=(v(i,j,k)+vbaro(i,j))*
     &                    max(onecm,dp(i,j,k)+dp(i,j-1,k))
            enddo !i
          enddo !j
c
          call usmoo(util1,work)
          call vsmoo(util2,work)
c
          do j=1,jj-1
            do i=2,ii-1
              u(i,j,k)=util1(i,j)/max(onecm,dp(i,j,k)+dp(i-1,j,k))
            enddo !i
          enddo !j
c
          do j=2,jj-1
            do i=1,ii-1
              v(i,j,k)=util2(i,j)/max(onecm,dp(i,j,k)+dp(i,j-1,k))
            enddo !i
          enddo !j
c
        enddo
c
        if (nintrp.le.1) then
c
c ---     no temporal interpolation
c
c         calculate dpu and dpv at u, v points.
c
          do j= 1,jj
            do i= 1,ii
              duk           = 0.0
              dvk           = 0.0
              depthi(:,:,0) = 0.0
              do k= 1,kk
                depthi(1,1,k) =
     &          depthi(1,1,k-1) + dp(    i,     j,     k)
                depthi(0,1,k) =
     &          depthi(0,1,k-1) + dp(max(i-1,1),j,     k)
                if(lperiod.and.i.eq.1) then
                  depthi(0,1,k) =
     &            depthi(0,1,k-1) + dp(idm,     j,     k)
                endif
                depthi(1,0,k) =
     &          depthi(1,0,k-1) + dp(    i, max(j-1,1),k)
                dukm1 = duk
                duk   = min( depthu(i,j),
     &                       0.5*(depthi(0,1,k) + depthi(1,1,k)) )
                dvkm1 = dvk
                dvk   = min( depthv(i,j),
     &                       0.5*(depthi(1,0,k) + depthi(1,1,k)) )
                dpu(i,j,k) = max( 0.0, duk-dukm1 )
                dpv(i,j,k) = max( 0.0, dvk-dvkm1 )
              enddo !k
            enddo !i
          enddo !j
c
          call floats_advect
c
        else
c
c ---     interpolate archives in time
c
c         fill alternate arrays for second archive.
c
          do j= 1,jj
            do i= 1,ii
              do k=1,kk
                temp2(i,j,k)=temp(i,j,k)
                saln2(i,j,k)=saln(i,j,k)
                th3d2(i,j,k)=th3d(i,j,k)
                u2(i,j,k)=u(i,j,k)
                v2(i,j,k)=v(i,j,k)
                dp2(i,j,k)=dp(i,j,k)
              enddo !k
            enddo !i
          enddo !j
c
          do jar=1,nintrp
c
            q=float(jar)/float(nintrp)
            if (q.lt.1) then
c
              if(jar.eq.1) then
                call getdat(flnm1,time1,iexpt,yrflag,trcout)
c
c ---           smooth grid-point noise in the velocity fields
                do k=1,kk
c
                  do j=1,jj-1
                    do i=2,ii-1
                      util1(i,j)=(u(i,j,k)+ubaro(i,j))*
     &                        max(onecm,dp(i,j,k)+dp(i-1,j,k))
                    enddo !i
                  enddo !j
c
                  do j=2,jj-1
                    do i=1,ii-1
                      util2(i,j)=(v(i,j,k)+vbaro(i,j))*
     &                            max(onecm,dp(i,j,k)+dp(i,j-1,k))
                    enddo !i
                  enddo !j
c
                  call usmoo(util1,work)
                  call vsmoo(util2,work)
c
                  do j=1,jj-1
                    do i=2,ii-1
                      u(i,j,k)=util1(i,j)/
     &                         max(onecm,dp(i,j,k)+dp(i-1,j,k))
                    enddo !i
                  enddo !j
c
                  do j=2,jj-1
                    do i=1,ii-1
                      v(i,j,k)=util2(i,j)/
     &                         max(onecm,dp(i,j,k)+dp(i,j-1,k))
                    enddo !i
                  enddo !j
                enddo !k
c
c               fill alternate arrays for first archive.
c
                do j= 1,jj
                  do i= 1,ii
                    do k=1,kk
                      temp1(i,j,k)=temp(i,j,k)
                      saln1(i,j,k)=saln(i,j,k)
                      th3d1(i,j,k)=th3d(i,j,k)
                      u1(i,j,k)=u(i,j,k)
                      v1(i,j,k)=v(i,j,k)
                      dp1(i,j,k)=dp(i,j,k)
                    enddo !k
                  enddo !i
                enddo !j
c
              endif !jar.eq.1
c
c ---         time interpolate
c
              do j=1,jj
                do i=1,ii
                  do k=1,kk
                    temp(i,j,k)=(1.0-q)*temp1(i,j,k)+q*temp2(i,j,k)
                    saln(i,j,k)=(1.0-q)*saln1(i,j,k)+q*saln2(i,j,k)
                    th3d(i,j,k)=(1.0-q)*th3d1(i,j,k)+q*temp2(i,j,k)
                    u(i,j,k)=(1.0-q)*u1(i,j,k)+q*u2(i,j,k)
                    v(i,j,k)=(1.0-q)*v1(i,j,k)+q*v2(i,j,k)
                    dp(i,j,k)=(1.0-q)*dp1(i,j,k)+q*dp2(i,j,k)
                  enddo !k
                enddo !i
              enddo !j
            else ! q.eq.1
              do j=1,jj
                do i=1,ii
                  do k=1,kk
                    temp(i,j,k)=temp2(i,j,k)
                    saln(i,j,k)=saln2(i,j,k)
                    th3d(i,j,k)=th3d2(i,j,k)
                    u(i,j,k)=u2(i,j,k)
                    v(i,j,k)=v2(i,j,k)
                    dp(i,j,k)=dp2(i,j,k)
                  enddo !k
                enddo !i
              enddo !j
c
            endif !q
c
c           calculate dpu and dpv at u, v points.
c
            do j= 1,jj
              do i= 1,ii
                duk           = 0.0
                dvk           = 0.0
                depthi(:,:,0) = 0.0
                do k= 1,kk
                  depthi(1,1,k) = 
     &            depthi(1,1,k-1) + dp(    i,     j,     k)
                  depthi(0,1,k) = 
     &            depthi(0,1,k-1) + dp(max(i-1,1),j,     k)
                  if(lperiod.and.i.eq.1) then
                    depthi(0,1,k) =
     &              depthi(0,1,k-1) + dp(idm,     j,     k)
                  endif
                  depthi(1,0,k) = 
     &            depthi(1,0,k-1) + dp(    i, max(j-1,1),k)
                  dukm1 = duk
                  duk   = min( depthu(i,j),
     &                         0.5*(depthi(0,1,k) + depthi(1,1,k)) )
                  dvkm1 = dvk
                  dvk   = min( depthv(i,j),
     &                         0.5*(depthi(1,0,k) + depthi(1,1,k)) )
                  dpu(i,j,k) = max( 0.0, duk-dukm1 )
                  dpv(i,j,k) = max( 0.0, dvk-dvkm1 )
                enddo !k
              enddo !i
            enddo !j
c
          call floats_advect
c
          enddo !jar
c
        endif !nintrp
c
      enddo  ! iar
c
      call floats_restart
c
      stop
c
      end
