      program floats
      use mod_floats ! HYCOM post-processing synthetic float interface
c
      implicit none
c
c --- post-processing lagrangian floats from a sequence of ascii file maps
c
      character*120    flnm1,flnm2
      character        aday*3,fn1*5,fn2*16
      logical          lexist,lfatal,trcout
      integer          kdmin,iinc,it,itr,jinc,jt,nntr,nasc,iday
      real             depthi(0:1,0:1,0:99),duk,dukm1,dvk,dvkm1
      real             ascin,ascfq
      real             scuxx,scuxy,scvyx,scvyy,factr,avglat
      real             uin(271,193),vin(271,193),tin(271,193)
      double precision time_ai,time_ao,time1,time2
c
      call xcspmd
c
      lperiod=.false.
      lp=6
      onecm=0.01*onem
      tencm=0.10*onem
c
      pi = 4.0*atan(1.0)
      radian=pi/180.0
c
c --- 'flnm_fii' = name of input floats input file
c --- 'flnm_fio' = name of output floats input file for next time segment 
c --- 'flnm_fo'  = name of floats output file containing path positions
c --- 'flnm_ai'  = name of first input ascii file
c --- 'flnm_ao'  = name of final input ascii file
c --- 'ascfq '   = no. of hours between ascii file input
c --- 'ascin '   = no. of hours between time-interpolated ascii file input
c --- 'idm   '   = longitudinal array size
c --- 'jdm   '   = latitudinal  array size
c --- 'nasc  '   = number of ascii files
c --- 'uvgrid'   = 0=u,v on p grid points; 1=u,v on staggered c grid
c
      read (*,'(a)') flnm_fii
      write (lp,'(2a)')
     & 'input floats input file:  ',flnm_fii(1:len_trim(flnm_fii))
      call flush(lp)
      read (*,'(a)') flnm_fio
      write (lp,'(2a)')
     & 'output floats input file: ',flnm_fio(1:len_trim(flnm_fio))
      call flush(lp)
      read (*,'(a)') flnm_fo
      write (lp,'(2a)')
     & 'floats output file:       ',flnm_fo(1:len_trim(flnm_fo))
      call flush(lp)
      read (*,'(a)') flnm_ai
      write (lp,'(2a)')
     & 'first ascii input file:   ',flnm_ai(1:len_trim(flnm_ai))
      call flush(lp)
      read (*,'(a)') flnm_ao
      write (lp,'(2a)')
     & 'last ascii input file:    ',flnm_ao(1:len_trim(flnm_ao))
      call flush(lp)
c
      call blkinr(ascfq,
     &           'ascfq ','("blkinr: ",a6," =",f11.4," hours")')
      call blkinr(ascin,
     &           'ascin ','("blkinr: ",a6," =",f11.4," hours")')
      ascfq=ascfq/24.0
      ascin=ascin/24.0
      nintrp=0
      if (ascfq.ne.ascin) then
        nintrp=nint(ascfq/ascin)
        if (ascfq/ascin.ne.float(nintrp)) then
          stop 'ascfq must be an integer multiple of ascin'
        endif
      endif
      archfq=ascfq
      archin=ascin
      call blkini(ii,    'idm   ')
      call blkini(jj,    'jdm   ')
      idm=ii
      jdm=jj
      kdm=5
      kk=5
      if     (ii.ne.idm .or. jj.ne.jdm) then
        write(lp,*)
        write(lp,*) 'error - wrong idm or jdm (should be:',idm,jdm,')'
        write(lp,*)
        call flush(lp)
        stop 
      endif
      call blkini(nasc, 'nasc  ')
      call blkini(iuvgrd, 'uvgrid')
c
c --- 'fltdbg' = FLOATS: float number to be debugged
c --- 'intpfl' = FLOATS: horiz. interp. (0=2nd order+bilinear; 1=bilinear)
c --- 'dfmonc' = FLOATS: evaporation/termination time scale (days for 50%)
c --- 'iturbv' = FLOATS: add horiz. turb. advection velocity (0=no; 1=yes)
c --- 'tbvar'  = FLOATS: horizontal turbulent velocity variance scale
c --- 'tdecri' = FLOATS: inverse decorrelation time scale
c 
      call blkini(nfl_debug,'fltdbg')
      call blkini(intpfl,'intpfl')
      call blkinr(dfmonc,'dfmonc','(a6," =",f10.4," days")')
      call blkini(iturbv,'iturbv')
      turbvel = iturbv.ge.1
      call blkinr(tbvar ,'tbvar ','(a6," =",f10.4," m**2/s**2")')
      call blkinr(tdecri,'tdecri','(a6," =",f10.4," 1/day")')
c
c --- array allocation
c
      call floats_alloc
c
      do j=1,jj
        do i=1,ii
          p(i,j,1)=0.0
          do k=1,kk
            dp(i,j,k)=10.0*9806.0
             p(i,j,k+1)=p(i,j,k)+dp(i,j,k)
          enddo !k
        enddo!i
      enddo !j
      do j=1,jj
        do i=1,ii
          depths(i,j)=50.0
        enddo !i
      enddo !j
c
      open (unit=nt1,file=flnm_fii,form='formatted',
     &      status='old',action='read' )
      open (unit=nt3,file=flnm_fo ,form='formatted',
     &      status='new',action='write')
c
c --- determine initial day and other file name segments
c
      aday=flnm_ai(6:8)
      read(aday,602) iday
  602 format(i3)
      read(flnm_ai,604) fn1
  604 format(a5)
      read(flnm_ai,606) fn2
  606 format(8x,a16)
c
c --- initialize the floats
c
      iar=1
      call flush(6)
      call getasc(flnm_ai,ii,jj,plon,plat,uin,vin,tin,depths)
c
      do j=1,jj
        do i=1,ii
          u(i,j,1)=uin(i,j)
          v(i,j,1)=vin(i,j)
          temp(i,j,1)=tin(i,j)
          if (depths(i,j).eq.0.0) then
            ip(i,j)=0
            do k=2,kk+1
              p(i,j,k)=0.0
            enddo !k
          else
            ip(i,j)=1
          endif !depths
        enddo !i
      enddo !j
c
c --- u,v points are located halfway between any 2 adjoining mass points
      do j=1,jj
        do i=1,ii
          if (ip(max(1,i-1),j).gt.0.and.ip(i,j).gt.0) then
            iu(i,j)=1
          else
            iu(i,j)=0
          endif !ip
          if (ip(i,max(1,j-1)).gt.0.and.ip(i,j).gt.0) then
            iv(i,j)=1
          else
            iv(i,j)=0
          endif !ip
        enddo !i
      enddo !j
c
      do j=1,jj
        do i=2,ii-1
          avglat=0.5*(plat(i-1,j)+plat(i,j))
          factr=cos(avglat*radian)
          scuxx=factr*(plon(i,j)-plon(i-1,j))
          scuxy=       plat(i,j)-plat(i-1,j)
          scux(i,j)=1.11e5*sqrt(scuxx*scuxx+scuxy*scuxy)
        enddo !i
        scux(1 ,j)=2.0*scux(2   ,j)-scux(3   ,j)
        scux(ii,j)=2.0*scux(ii-1,j)-scux(ii-2,j)
      enddo !j
c
      do i=1,ii
        do j=2,jj-1
          avglat=0.5*(plat(i,j-1)+plat(i,j))
          factr=cos(avglat*radian)
          scvyx=factr*(plon(i,j)-plon(i,j-1))
          scvyy=       plat(i,j)-plat(i,j-1)
          scvy(i,j)=1.11e5*sqrt(scvyx*scvyx+scvyy*scvyy)
        enddo !j
        scvy(i, 1)=2.0*scvy(i,   2)-scvy(i,   3)
        scvy(i,jj)=2.0*scvy(i,jj-1)-scvy(i,jj-2)
      enddo !i
c
c --- smooth grid-point noise in the model fields (disabled)
c
c     call usmoo(u(:,:,1),work)
c     call vsmoo(v(:,:,1),work)
c
      do k=2,kk
        do i=1,ii
          do j=1,jj
            u(i,j,k)=u(i,j,1)
            v(i,j,k)=v(i,j,1)
            temp(i,j,k)=temp(i,j,1)
          enddo !j
        enddo !i
      enddo !k
c
      call floats_init
c     stop 'for debug, stop at end of floats_init'
c
c --- initial call to floats_advect writes out float info. at initial time
c
      call floats_advect
c
      flnm1=flnm_ai
      flnm2=flnm_ai
c
      do iar=2,nasc 
c
c ---   loop through all input model ascii files
c
        iday=iday+1
        if (nintrp.gt.1) then
          write(flnm1,902) fn1,iday-1,fn2
  902     format(a5,i3.3,a16)
        endif !nintrp
        write(flnm2,902) fn1,iday,fn2
        call getasc(flnm2,ii,jj,plon,plat,uin,vin,tin,depths)
        do j=1,jj
          do i=1,ii
            u2(i,j,1)=uin(i,j)
            v2(i,j,1)=vin(i,j)
            temp2(i,j,1)=tin(i,j)
          enddo !i
        enddo !j
c
c --- smooth grid-point noise in the model fields (disabled)
c
c       call usmoo(u2(:,:,1),work)
c       call vsmoo(v2(:,:,1),work)
c
        do k=2,kk
          do i=1,ii
            do j=1,jj
              u2(i,j,k)=u2(i,j,1)
              v2(i,j,k)=v2(i,j,1)
              temp2(i,j,k)=temp2(i,j,1)
            enddo !j
          enddo !i
        enddo !k
c
        if (nintrp.le.1) then
c
c ---     no temporal interpolation
c
          do k=1,kk
            do i=1,ii
              do j=1,jj
                u(i,j,k)=u2(i,j,k)
                v(i,j,k)=v2(i,j,k)
                temp(i,j,k)=temp2(i,j,k)
              enddo !j
            enddo !i
          enddo !k
          call floats_advect
c
        else
c
c ---     interpolate ascii files in time
c
          do jar=1,nintrp
c
            q=float(jar)/float(nintrp)
            if (q.lt.1) then
c
              if(jar.eq.1) then
                call getasc(flnm1,ii,jj,plon,plat,uin,vin,tin,depths)
                do j=1,jj
                  do i=1,ii
                    u1(i,j,1)=uin(i,j)
                    v1(i,j,1)=vin(i,j)
                    temp1(i,j,1)=tin(i,j)
                  enddo !i
                enddo !j
c
c ---           smooth grid-point noise in the model fields (disabled)
c
c               call usmoo(u(:,:,1),work)
c               call vsmoo(v(:,:,1),work)
              endif
c
              do k=2,kk
                do i=1,ii
                  do j=1,jj
                    u1(i,j,k)=u1(i,j,1)
                    v1(i,j,k)=v1(i,j,1)
                    temp1(i,j,k)=temp1(i,j,1)
                  enddo !j
                enddo !i
              enddo !k
c
c ---         time interpolate
c
              do k=1,kk
                do j=1,jj
                  do i=1,ii
                    u(i,j,k)=(1.0-q)*u1(i,j,k)+q*u2(i,j,k)
                    v(i,j,k)=(1.0-q)*v1(i,j,k)+q*v2(i,j,k)
                    temp(i,j,k)=(1.0-q)*temp1(i,j,k)+q*temp2(i,j,k)
                  enddo !i
                enddo !j
              enddo !k
            else
              do k=1,kk
                do j=1,jj
                  do i=1,ii
                    u(i,j,k)=u2(i,j,k)
                    v(i,j,k)=v2(i,j,k)
                    temp(i,j,k)=temp2(i,j,k)
                  enddo !i
                enddo !j
              enddo !k
c
            endif !q
c
            call floats_advect
c
          enddo !jar
c
        endif !nintrp
c
      enddo  ! iar
c
      call floats_restart
c
      stop
c
      end
c
c
      subroutine getasc(fname,ii,jj,alon,alat,uin,vin,tin,dep)
c
c --- subroutine to read ascii u,v files, reset land values (999) to zero
c
      character*120 fname
      integer ii,jj
      real uin(ii,jj),vin(ii,jj),tin(ii,jj),dep(ii,jj)
      real alon(ii,jj),alat(ii,jj)
c
      write(6,900) fname
  900 format('file opened for reading:'/,120a)
      open(1,file=fname,readonly,status='old')
c
      do j=1,jj
        do i=1,ii
          read(1,*) alon(i,j),alat(i,j),uin(i,j),vin(i,j),tin(i,j)
          if(abs(uin(i,j)).gt.90.0 .or. abs(vin(i,j)).gt.90.0) then
            tin(i,j)=999.0
            uin(i,j)=999.0
            vin(i,j)=999.0
            dep(i,j)=0.0
          endif
        enddo
      enddo
c
      close(1)
c
      return
      end
