      module mod_floats
      use mod_xc    ! HYCOM communication interface
c
      implicit none
c
c --- HYCOM synthetic floats and drifters
c --- See subroutine floats (below) for more information
c
c --- maximum number of floats limited only by available memory
c
      integer, parameter, public :: 
     & nfldim=6000  !maximum number of synthetic floats
c
      character*120 flnm_fii,flnm_fio,flnm_fo,flnm_ai,flnm_ao
c
      integer, save :: i,j,k,kdm,ii,jj,kk,iar,jar,narch
     &                ,nfl_debug,iuvgrd
c
c --- ms-1  = max. number of interruptions of any grid row or column by land
c
      integer, parameter :: ms=99
c
      real,         save, allocatable, dimension (:,:,:) :: 
     &   temp,saln,th3d,u,v,p,dp,pu,dpu,pv,dpv,
     &   temp1,saln1,th3d1,u1,v1,dp1,
     &   temp2,saln2,th3d2,u2,v2,dp2

      real,         save, allocatable, dimension (:,:)   ::
     &   ubaro,vbaro,depths,depthu,depthv,depth1,
     &   scpx,scpy,scux,scuy,scvx,scvy,scu2,scv2,
     &   plon,plat,plonf,platf,
     &   ulon,ulat,ulonf,ulatf,vlon,vlat,vlonf,vlatf 
c
      real,    allocatable, dimension(:,:,:),
     &         save, public ::
     & uold2,
     & vold2,
     & wold2u,
     & wold2d

      real,    allocatable, dimension(:,:),
     &         save, public ::
     & dlondx,
     & dlondy,
     & dlatdx,
     & dlatdy,
     & dpdx,
     & dpdy,
     & util1,
     & util2,
     & util3,
     & util4,
     & work
c
      real,    allocatable, dimension(:,:,:),
     &         save, public ::
     & wvelup,wveldn

      real,    save :: flt(nfldim,9),archfq,archin,onecm,tencm,thbase,
     &                 deltfl,fldepm,tbvar,tdecri,dtturb,uturb0,
     &                 dudx,dvdy,timefl,cfmonc,cfest,dfmonc,radian,pi,q

      integer, save :: kfloat(nfldim),iflnum(nfldim),ifltyp(nfldim),
     &                 intpfl,iturbv,ismpfl,nfl,nflt,nstepin,nstepfl,
     &                 nintrp
c
      integer, save, allocatable, dimension (:,:)   ::
     &   ip,iq,iu,iv,
     &   ifp,ilp,jfp,jlp,
     &   ifq,ilq,jfq,jlq,
     &   ifu,ilu,jfu,jlu,
     &   ifv,ilv,jfv,jlv,
     &   ifd,ild,jfd,jld
c
      integer, save, allocatable, dimension (:)     ::
     &   isp,jsp, isq,jsq, isu,jsu, isv,jsv, nsec
c
      logical, save :: turbvel,wvelfl,nonlatlon,lperiod

      real,    parameter   :: huge=1.0e20
      real,    parameter   :: epsil=1.0e-12
      real,    parameter   :: onem=9806.0

      integer, parameter   :: nt1=21           ! input floats input file
      integer, parameter   :: nt2=22           ! output floats input file
      integer, parameter   :: nt3=23           ! floats output file

c --- coefficients for sigma-0 (based on Brydon & Sun fit)
      real, parameter ::
     &  c10=-1.36471E-01,  !const. coefficent
     &  c20= 4.68181E-02,  !T      coefficent
     &  c30= 8.07004E-01,  !   S   coefficent
     &  c40=-7.45353E-03,  !T^2    coefficent
     &  c50=-2.94418E-03,  !T  S   coefficent
     &  c60= 3.43570E-05,  !T^3    coefficent
     &  c70= 3.48658E-05   !T^2S   coefficent

c --- coefficients for sigma-2 (based on Brydon & Sun fit)
      real, parameter ::
     &  c12= 9.77093E+00,  !const. coefficent
     &  c22=-2.26493E-02,  !T      coefficent
     &  c32= 7.89879E-01,  !   S   coefficent
     &  c42=-6.43205E-03,  !T^2    coefficent
     &  c52=-2.62983E-03,  !T  S   coefficent
     &  c62= 2.75835E-05,  !T^3    coefficent
     &  c72= 3.15235E-05   !T^2S   coefficent

      real, save :: c1,c2,c3,c4,c5,c6,c7

      contains

      subroutine floats_alloc
      implicit none
c
c --- initialize allocatable arrays.
c
      allocate(   temp(ii,jj,kk  ) )
      allocate(   saln(ii,jj,kk  ) )
      allocate(   th3d(ii,jj,kk  ) )
      allocate(      u(ii,jj,kk  ) )
      allocate(      v(ii,jj,kk  ) )
      allocate(      p(ii,jj,kk+1) )
      allocate(     dp(ii,jj,kk  ) )
      allocate(     pu(ii,jj,kk+1) )
      allocate(    dpu(ii,jj,kk  ) )
      allocate(     pv(ii,jj,kk+1) )
      allocate(    dpv(ii,jj,kk  ) )
c
      allocate(   temp1(ii,jj,kk  ) )
      allocate(   saln1(ii,jj,kk  ) )
      allocate(   th3d1(ii,jj,kk  ) )
      allocate(      u1(ii,jj,kk  ) )
      allocate(      v1(ii,jj,kk  ) )
      allocate(     dp1(ii,jj,kk  ) )
c
      allocate(   temp2(ii,jj,kk  ) )
      allocate(   saln2(ii,jj,kk  ) )
      allocate(   th3d2(ii,jj,kk  ) )
      allocate(      u2(ii,jj,kk  ) )
      allocate(      v2(ii,jj,kk  ) )
      allocate(     dp2(ii,jj,kk  ) )
c
      allocate(  ubaro(ii,jj) )
      allocate(  vbaro(ii,jj) )
      allocate( depth1(0:ii,0:jj) )
      allocate( depths(ii,jj) )
      allocate( depthu(ii,jj) )
      allocate( depthv(ii,jj) )
      allocate(   plon(ii,jj) )
      allocate(   plat(ii,jj) )
      allocate(   ulon(ii,jj) )
      allocate(   ulat(ii,jj) )
      allocate(   vlon(ii,jj) )
      allocate(   vlat(ii,jj) )
      allocate(  plonf(-2:ii+3,jj) )
      allocate(  platf(-2:ii+3,jj) )
      allocate(  ulonf(-2:ii+3,jj) )
      allocate(  ulatf(-2:ii+3,jj) )
      allocate(  vlonf(-2:ii+3,jj) )
      allocate(  vlatf(-2:ii+3,jj) )
      allocate(   scpx(ii,jj) )
      allocate(   scpy(ii,jj) )
      allocate(   scux(ii,jj) )
      allocate(   scuy(ii,jj) )
      allocate(   scvx(ii,jj) )
      allocate(   scvy(ii,jj) )
      allocate(   scu2(ii,jj) )
      allocate(   scv2(ii,jj) )
      allocate(  util1(ii,jj) )
      allocate(  util2(ii,jj) )
      allocate(  util3(ii,jj) )
      allocate(  util4(ii,jj) )
      allocate(   work(ii,jj) )
c
      allocate(     ip(ii,jj) )
      allocate(     iq(ii,jj) )
      allocate(     iu(ii,jj) )
      allocate(     iv(ii,jj) )
c
      allocate(    ifp(jj,ms) )
      allocate(    ilp(jj,ms) )
      allocate(    jfp(ii,ms) )
      allocate(    jlp(ii,ms) )
      allocate(    ifq(jj,ms) )
      allocate(    ilq(jj,ms) )
      allocate(    jfq(ii,ms) )
      allocate(    jlq(ii,ms) )
      allocate(    ifu(jj,ms) )
      allocate(    ilu(jj,ms) )
      allocate(    jfu(ii,ms) )
      allocate(    jlu(ii,ms) )
      allocate(    ifv(jj,ms) )
      allocate(    ilv(jj,ms) )
      allocate(    jfv(ii,ms) )
      allocate(    jlv(ii,ms) )
c
      allocate(    isp(jj) )
      allocate(    jsp(ii) )
      allocate(    isq(jj) )
      allocate(    jsq(ii) )
      allocate(    isu(jj) )
      allocate(    jsu(ii) )
      allocate(    isv(jj) )
      allocate(    jsv(ii) )
c
      allocate( wvelup(ii,jj,  kdm) )
      allocate( wveldn(ii,jj,  kdm) )
      allocate(  uold2(ii,jj,2*kdm) )
      allocate(  vold2(ii,jj,2*kdm) )
      allocate( wold2u(ii,jj,2*kdm) )
      allocate( wold2d(ii,jj,2*kdm) )
      allocate( dlondx(ii,jj      ) )
      allocate( dlondy(ii,jj      ) )
      allocate( dlatdx(ii,jj      ) )
      allocate( dlatdy(ii,jj      ) )
      allocate(   dpdx(ii,jj      ) )
      allocate(   dpdy(ii,jj      ) )

      end subroutine floats_alloc

      subroutine floats_init
c
c --- read initial float data
c --- initialize parameters and arrays required for floats
c
      integer i,j,k,l
      integer nbcday,nsmday,ityp
c
c --- initialize flags
c
c --- wvelfl: calculate vertical velocity?
      wvelfl=.false.
c
c --- determine if this is a global periodic grid
c
c     lperiod = maxval(plon(:,:))-minval(plon(:,:)) .gt. 350.0
c
c --- nonlatlon: does the model grid cross latitude/longitude lines anywhere
c --- in the domain?
      nonlatlon=.false.
c
c --- calculate the time interval for the runga-kutta interpolation
c --- as archin, the time interval between archive files (after time
c --- interpolation if requested) which is also the float advection
c --- time interval
      deltfl=archin*86400.0
c
c --- set minimum float depth (m)
      fldepm=1.0
c
c --- -------------------------------------------------------------
c --- initialize synthetic drifters, floats, and moored instruments
c --- -------------------------------------------------------------
c
      open(unit=nt1,file=flnm_fii,status='old')
c
c --- first line in float input file is the number of floats
      read(nt1,*) nflt
      if (nflt.gt.nfldim) then
        write(lp,'(/ a /)')
     &    'error - increase nfldim in mod_floats.F'
        call flush(lp)
        call xcstop('(floats_init)')
               stop '(floats_init)'
      endif
c
c --- second line in float input file is the initial float time step
c --- this number is zero for the initial model run segment
      read(nt1,*) nstepin
      timefl=nstepin*archin
      nstepfl=nstepin-1
c
      do nfl=1,nflt
c
c --- read input file containing the following information for each float:
c ---
c ---   column 1 - initial sequential float number
c ---   column 2 - float type
c ---              1 = 3-d lagrangian (vertically advected by diagnosed w)
c ---              2 = isopycnic
c ---              3 = isobaric (surface drifter when released in sfc. layer)
c ---   column 3 - deployment time (days from model start, 0.0 = immediate)
c ---   column 4 - termination time (days from model start, 0.0 = forever)
c ---   column 5 - initial longitude (must be between minimum and maximum
c ---                                 longitudes defined in regional.grid.b)
c ---   column 6 - initial latitude  (must be between minimum and maximum
c ---                                 latitudes defined in regional.grid.b)
c ---   column 7 - initial depth (or reference sigma for isopycnic floats)
c
        read(nt1,*) iflnum(nfl),ifltyp(nfl),flt(nfl,6),flt(nfl,7),
     &              flt(nfl,1),flt(nfl,2),flt(nfl,3)
        if (ifltyp(nfl).eq.1 .and. iuvgrd.eq.1) then ! hycom c grid only
          wvelfl=.true.
        endif
        flt(nfl,8)=0.0
        flt(nfl,9)=0.0
c
        if(ifltyp(nfl).eq.2) then
          flt(nfl,8)=flt(nfl,3)
          flt(nfl,3)=0.0
        endif
c
        kfloat(nfl)=-1
c
      enddo
c
      close(unit=nt1) !file='float.input'
c
      write(lp,955) nflt
      write(lp,956) min(nflt,10)
  955 format(' read initial data for',i6,' floats, time step',i9)
  956 format(' input data for first',i3,' floats')
      do nfl=1,min(nflt,10)
        write(lp,957) nfl,(flt(nfl,i),i=1,9)
  957   format(3x,i2,1p,5e13.5/5x,4e13.5/)
      enddo
      call flush(lp)
c
      do j=1,jj
        do i=1,ii
          plonf(i,j)=plon(i,j)
          ulonf(i,j)=ulon(i,j)
          vlonf(i,j)=vlon(i,j)
          platf(i,j)=plat(i,j)
          ulatf(i,j)=ulat(i,j)
          vlatf(i,j)=vlat(i,j)
          do k=1,kk
            uold2( i,j,k   )=huge
            uold2( i,j,k+kk)=huge
            vold2( i,j,k   )=huge
            vold2( i,j,k+kk)=huge
            wold2u( i,j,k   )=huge
            wold2u( i,j,k+kk)=huge
            wold2d( i,j,k   )=huge
            wold2d( i,j,k+kk)=huge
          enddo !k
        enddo !i
        if (lperiod) then
          do i=-2,0
            plonf(i,j)=plon(i+ii,j)-360.0
            ulonf(i,j)=ulon(i+ii,j)-360.0
            vlonf(i,j)=vlon(i+ii,j)-360.0
            platf(i,j)=plat(i+ii,j)
            ulatf(i,j)=ulat(i+ii,j)
            vlatf(i,j)=vlat(i+ii,j)
          enddo !i
          do i=ii+1,ii+3
            plonf(i,j)=plon(i-ii,j)+360.0
            ulonf(i,j)=ulon(i-ii,j)+360.0
            vlonf(i,j)=vlon(i-ii,j)+360.0
            platf(i,j)=plat(i-ii,j)
            ulatf(i,j)=ulat(i-ii,j)
            vlatf(i,j)=vlat(i-ii,j)
          enddo !i
        else
          do i=-2,0
            plonf(i,j)=plon(1,j)
            ulonf(i,j)=ulon(1,j)
            vlonf(i,j)=vlon(1,j)
            platf(i,j)=plat(1,j)
            ulatf(i,j)=ulat(1,j)
            vlatf(i,j)=vlat(1,j)
          enddo !i
          do i=ii+1,ii+3
            plonf(i,j)=plon(ii,j)
            ulonf(i,j)=ulon(ii,j)
            vlonf(i,j)=vlon(ii,j)
            platf(i,j)=plat(ii,j)
            ulatf(i,j)=ulat(ii,j)
            vlatf(i,j)=vlat(ii,j)
          enddo !i
        endif !lperiod
      enddo !j
c
c ---------------------------------------------------------------------------
c --- initialize old horizontal velocities for runga-kutta time interpolation
c --- calculate dlondx, dlondy, dlatdx, dlatdy required to convert u, v to
c --- longitude/time and latitude/time for float advection
c ---------------------------------------------------------------------------
c
      do j=1,jj
c
        do i=1,ii
          if (iu(i,j).eq.1) then
            do k=1,kk
              uold2(i,j,k   )=u(i,j,k)
              uold2(i,j,k+kk)=u(i,j,k)
            enddo
          endif
c
          if (iv(i,j).eq.1) then
            do k=1,kk
              vold2(i,j,k   )=v(i,j,k)
              vold2(i,j,k+kk)=v(i,j,k)
            enddo
          endif
c
          if (ip(i,j).eq.1) then
            do k=1,kk
              wold2u(i,j,k   )=0.0
              wold2u(i,j,k+kk)=0.0
              wold2d(i,j,k   )=0.0
              wold2d(i,j,k+kk)=0.0
            enddo
          endif
        enddo
c
      enddo !j
c
      do j=1,jj
c
        do i=ii,1,-1
          if (i.ge.2) then
            dlondx(i,j)=(plon(i,j)-plon(i-1,j))/scux(i,j)
            dlatdx(i,j)=(plat(i,j)-plat(i-1,j))/scux(i,j)
            if (abs(plat(i,j)-plat(i-1,j)).gt.1.0e-9) then
              nonlatlon=.true.
            endif
          else
            if (lperiod) then
              dlondx(1,j)=(plon(1,j)-plon(ii,j)-360.0)/scux(i,j)
              dlatdx(1,j)=(plat(1,j)-plat(ii,j)-360.0)/scux(i,j)
            else
              if     (iu(2,j).eq.1 .and. iu(3,j).eq.1) then
                dlondx(1,j)=2.0*dlondx(2,j)-dlondx(3,j)
                dlatdx(1,j)=2.0*dlatdx(2,j)-dlatdx(3,j)
              elseif (iu(2,j).eq.1) then
                dlondx(1,j)=dlondx(2,j)
                dlatdx(1,j)=dlatdx(2,j)
              endif
            endif !lperiod
          endif !i.ge.2
        enddo !i
c
      enddo !j
c
      do i=1,ii
c
        do j=jj,1,-1
          if (j.ge.2) then
            dlondy(i,j)=(plon(i,j)-plon(i,j-1))/scvy(i,j)
            dlatdy(i,j)=(plat(i,j)-plat(i,j-1))/scvy(i,j)
            if (abs(plon(i,j)-plon(i,j-1)).gt.1.0e-9) then
              nonlatlon=.true.
            endif
          else
            if     (iv(i,2).eq.1 .and. iv(i,3).eq.1) then
              dlondy(i,1)=2.0*dlondy(i,2)-dlondy(i,3)
              dlatdy(i,1)=2.0*dlatdy(i,2)-dlatdy(i,3)
            elseif (iv(i,2).eq.1) then
              dlondy(i,1)=dlondy(i,2)
              dlatdy(i,1)=dlatdy(i,2)
            endif
          endif !j.ge.2
        enddo !j
c
      enddo !i
c
c --- interpolate dlondx, dlondy, dlatdx, dlatdy to p points for non-hycom,
c --- non-staggered grid
c
      if (iuvgrd.eq.0) then  ! non-hycom grid
        do j=1,jj
          do i=1,ii-1
            util1(i,j)=0.5*(dlondx(i,j)+dlondx(i+1,j))
            util2(i,j)=0.5*(dlatdx(i,j)+dlatdx(i+1,j))
          enddo !i
          util1(ii,j)=util1(ii-1,j)
          util2(ii,j)=util2(ii-1,j)
        enddo !j
        do i=1,ii
          do j=1,jj-1
            util3(i,j)=0.5*(dlondy(i,j)+dlondy(i,j+1))
            util4(i,j)=0.5*(dlatdy(i,j)+dlatdy(i,j+1))
          enddo !j
          util3(ii,j)=util3(ii,j-1)
          util4(ii,j)=util4(ii,j-1)
        enddo !i
        do j=1,jj
          do i=1,ii
            dlondx(i,j)=util1(i,j)
            dlatdx(i,j)=util2(i,j)
            dlondy(i,j)=util3(i,j)
            dlatdy(i,j)=util4(i,j)
          enddo !i
        enddo !j
      endif ! iuvgrd.eq.0
c
c ----------------------------------------
c --- turbulent horizontal velocity option
c ----------------------------------------
c
c --- initialize turbulence constants
c
c --- tdecri is in inverse days and dtturb is the turbulent time step in days
c
c --- dtturb is set to the runga-kutta interpolation time step, which must
c --- be within 1 and 2 hours
c
      if (turbvel) then
        dtturb=deltfl/86400.0
        uturb0=sqrt(2.*tbvar*tdecri*dtturb)
      endif
c
      return
      end subroutine floats_init

      subroutine floats_restart
c
c -----------------------------
c --- output float restart file
c -----------------------------
c
      integer i,k,l
c
c ---   first remove floats that have run aground or left the domain
      i=0
      do l=1,nflt
        if (flt(l,1).gt.-999.0) then
          i=i+1
        endif
      enddo !1
c
      write(lp,*) 'writing new float input file for restart'
      call flush(lp)
c
      open(unit=nt2,file=flnm_fio,form='formatted',status='new')
      write(nt2,970) i
      write(nt2,970) nstepfl
      do l=1,nflt
        if (flt(l,1).gt.-999.0) then
          if (ifltyp(l).ne.2) then
            write(nt2,971) iflnum(l),ifltyp(l),flt(l,4),flt(l,5),
     &                   (flt(l,k),k=1,3)
          else
            write(nt2,971) iflnum(l),ifltyp(l),flt(l,4),flt(l,5),
     &                   (flt(l,k),k=1,2),flt(l,6)
          endif
        endif
      enddo !l
      close(nt2)
c
  970 format(i9)
  971 format(i9,i2,2f10.3,3f15.9)

      return
      end subroutine floats_restart

      subroutine floats_advect
c
c --- local variables
      integer l
      real    tflt,sflt,thlo,thhi,
     &        uflt1,uflt2,uflt3,uflt4,uflt,
     &        vflt1,vflt2,vflt3,vflt4,vflt,
     &        wflt1,wflt2,wflt3,wflt4,wflt,
     &        vkflt,tkflt,skflt,xpos0,xpos1,xpos2,xpos3,
     &        ypos0,ypos1,ypos2,ypos3,depflt,plo,phi,q,
     &        depiso,wvhi,wvlo,uvfctr,qisop,ufltm,vfltm,
     &        alomin,alomax,alamin,alamax,
     &        ulo1,ulo2,ulo3,ulo4,vlo1,vlo2,vlo3,vlo4,
     &        ula1,ula2,ula3,ula4,vla1,vla2,vla3,vla4
      real    uturb,vturb,uturb1,vturb1,dlodx,dlody,dladx,dlady
      real*4  rtab(200,2)
      integer ifltll(3),jfltll(3),ngood(3)
      integer kflt,k1,k2,k3,ier,ier2,ngrid,kold1,kold2,
     &        ntermn,i1,j1,ngoodi
      integer ied,iede,ifladv
c
      integer*4 seed,numran,inisee,iflag
      integer*2 iseed1,iseed2
      equivalence ( iseed1, iseed2, seed ) 
c
c --- 2-d local arrays used for the 16-point box horizontal interpolation
      real    varb2d(4,4),ptlon(4,4,3),ptlat(4,4,3)
      logical maskpt(4,4,3),maskpi(4,4),masktopo(4,4)
c
      real   timer1, timer2, totime
      real*8 wtime
c
c --- equation of state
      real   sig0,sig2
      real   t,s
      sig0(t,s)=(c10+c30*s+t*(c20+c50*s+t*(c40+c70*s+c60*t)))
      sig2(t,s)=(c12+c32*s+t*(c22+c52*s+t*(c42+c72*s+c62*t)))
c
c -------------------------------------------------------------------------
c --- floats.f (post-processing version)
c -------------------------------------------------------------------------
c
c --- synthetic floats, drifters, and mooring instruments
c
c --- optionally samples time series of dynamical/thermodynamical variables
c --- at the location of each float
c
c --- three float types are presently supported:
c ---    3-d lagrangian (vertically advected by diagnosed vertical velocity)
c ---    isopycnic      (remains on specified density surface)
c ---    isobaric       (remains at the pressure depth where it was released)
c
c --- horizontal advection, along with vertical advection of lagrangian floats,
c --- is performed using the MICOM runga-kutta-4 time interpolation algorithm
c --- developed by Zulema Garraffo
c
c --- works on any curvilinear grid
c
c --- float position is stored as longitude and latitude
c
c --- to horizontal advect the float, u and v are first converted to
c --- d(longitude)/dt and d(latitude)/dt as follows:
c
c ---   u_lon = u*dlondx + v*dlondy
c ---   v_lat = u*dlatdx + v*dlatdy
c
c --- to horizontally advect the float, terms u*dlondx and u*dlatdx are
c --- calculated on u grid points while terms v*dlondy and v*dlatdy are
c --- calculated on v grid ponts. these terms are then interpolated to
c --- the float locations from the surrounding u and v grid points,
c --- respectively
c
c --- since the terms v*dlondy and u*dlatdx are always zero when the model
c --- x,y axes are everywhere lines of constant latitude and longitude,
c --- respectively, logical variable 'nonlatlon' prevents horizontal
c --- interpolation of these terms in this case
c
c --- horizontal interpolation to the float location follows the MICOM
c --- procedure of Zulema Garraffo - second-order interpolation from the 16
c --- surrounding grid points is performed unless fewer than nptmin (presently
c --- set to 9 in subroutine intrph) water points are available, in which
c --- case nearest-neighbor interpolation is used.
c
c --- variables are horizontally interpolated from their native grid (p, u,
c --- or v)
c
c --- adapted for hycom by george halliwell
c --- code parallelized by remy baraille
c
c --- the second-order interpolation subroutine included here is the one
c --- included in the mariano and brown parameter matrix objective analysis
c --- algorithm to estimate the large scale trend surface - it is different
c --- from the algorithm used by Garraffo in MICOM
c
c --- float initialization is performed in floats_init - information
c --- about the input file containing initial float information is
c --- presented in that subroutine.
c
c --- variables in array flt(nfl,n) are:
c
c ---   n = 1  longitude
c ---   n = 2  latitude
c ---   n = 3  float depth 
c ---   n = 4  float start time (in days from start of model run)
c ---   n = 5  float end   time (in days from start of model run)
c ---   n = 6  float target density surface (isopycnic floats only)
c
c --- variables output onto file float.out for each float are:
c
c ---   1. initial sequential float number
c ---   2. time (in days from start of model run)
c ---   3. model layer number that contains the float
c ---   4. longitude
c ---   5. latitude
c ---   6. float depth
c
      nstepfl=nstepfl+1
      timefl=nstepfl*archin
c
c --- set pressure arrays at p, u, and v points
c --- calculate th3d array
c --- calculate total velocity components
      do j=1,jj
        do i=1,ii
          if (ip(i,j).eq.1) then
            do k=1,kk
              p(i,j,k+1)=p(i,j,k)+dp(i,j,k)
              if (thbase.lt.30.0) then
                th3d(i,j,k)=sig0(temp(i,j,k),saln(i,j,k))
              else
                th3d(i,j,k)=sig2(temp(i,j,k),saln(i,j,k))
              endif
            enddo
          endif
          if (iu(i,j).eq.1) then
            do k=1,kk
              pu(i,j,k+1)=pu(i,j,k)+dpu(i,j,k)
            enddo
          endif
          if (iv(i,j).eq.1) then
            do k=1,kk
              pv(i,j,k+1)=pv(i,j,k)+dpv(i,j,k)
            enddo
          endif
        enddo
      enddo
c
c --- fluid vertical velocity
c --- wvelup(*,*,k) is the vertical velocity at the top    of layer k
c --- wveldn(*,*,k) is the vertical velocity at the bottom of layer k
      if (wvelfl) then
        do j= 1,jj-1
          do i= 1,ii-1
            if (ip(i,j).eq.1) then
              dudx=(u(i+1,j  ,1)*scuy(i+1,j  )-u(i,j,1)*scuy(i,j))
     &             /(scpx(i,j)*scpy(i,j))
              dvdy=(v(i  ,j+1,1)*scvx(i  ,j+1)-v(i,j,1)*scvx(i,j))
     &             /(scpx(i,j)*scpy(i,j))
              wvelup(i,j,1)=0.0
              wveldn(i,j,1)=dp(i,j,1)*(dudx+dvdy)/onem
            endif
          enddo !i
          if (lperiod) then
            i=ii   
            if     (ip(i,j).eq.1) then
              dudx=(u(1  ,j  ,1)*scuy(1  ,j  )-u(i,j,1)*scuy(i,j))
     &             /(scpx(i,j)*scpy(i,j))
              dvdy=(v(i  ,j+1,1)*scvx(i  ,j+1)-v(i,j,1)*scvx(i,j))
     &             /(scpx(i,j)*scpy(i,j))
              wvelup(i,j,1)=0.0
              wveldn(i,j,1)=dp(i,j,1)*(dudx+dvdy)/onem
            endif
          endif
        enddo !j
c
        do k=2,kk
          do j= 1,jj
            do i= 2,ii-1
              if     (iu(i,j).eq.1) then
                dpdx(i,j)=(p(i,j,k)-p(i-1,j  ,k))/scux(i,j)
              endif
            enddo
            if (lperiod) then
              if (iu(1,j).eq.1) then
                dpdx(1 ,j)=(p(1 ,j,k)-p(ii  ,j,k))/scux(1,j)
              endif
              if (iu(ii,j).eq.1) then
                dpdx(ii,j)=(p(ii,j,k)-p(ii-1,j,k))/scux(ii,j)
              endif
            endif
          enddo
          do i= 1,ii
            do j= 2,jj-1
              if     (iv(i,j).eq.1) then
                dpdy(i,j)=(p(i,j,k)-p(i  ,j-1,k))/scvy(i,j)
              endif
            enddo
          enddo
c
          if (.not.lperiod) then
            do j=1,jj
              if     (iu(1   ,j).eq.1 .and. iu(2   ,j).eq.1) then
                dpdx(1 ,j)=dpdx(2   ,j)
              endif
              if     (iu(ii-1,j).eq.1) then
                dpdx(ii,j)=dpdx(ii-1,j)
              endif
            enddo
            do i=1,ii
              if     (iv(i,2   ).eq.1) then
                dpdy(i,1 )=dpdy(i,2   )
              endif
              if     (iv(i,jj-1).eq.1) then
                dpdy(i,jj)=dpdy(i,jj-1)
              endif
            enddo
          endif
          do j= 1,jj-1
            do i= 1,ii-1
              if     (ip(i,j).eq.1) then
                dudx=(u(i+1,j  ,k  )*scuy(i+1,j  )-
     &                u(i  ,j  ,k  )*scuy(i  ,j  ))/
     &               (scpx(i,j)*scpy(i,j))
                dvdy=(v(i  ,j+1,k  )*scvx(i  ,j+1)-
     &                v(i  ,j  ,k  )*scvx(i  ,j  ))/
     &               (scpx(i,j)*scpy(i,j))
                if (dp(i,j,k).gt.0.0) wvelup(i,j,k)=wveldn(i,j,k-1)-0.5*
     &                ((u(i  ,j  ,k)-u(i  ,j  ,k-1))*dpdx(i  ,j  )+
     &                 (u(i+1,j  ,k)-u(i+1,j  ,k-1))*dpdx(i+1,j  )+
     &                 (v(i  ,j  ,k)-v(i  ,j  ,k-1))*dpdy(i  ,j  )+
     &                 (v(i  ,j+1,k)-v(i  ,j+1,k-1))*dpdy(i  ,j+1))
     &                 /onem
                wveldn(i,j,k)=wvelup(i,j,k  )+dp(i,j,k)*(dudx+dvdy)/onem
              endif
            enddo !i
            if (lperiod) then
              i=ii
              if     (ip(i,j).eq.1) then
                dudx=(u(1  ,j  ,k  )*scuy(1  ,j  )-
     &                u(i  ,j  ,k  )*scuy(i  ,j  ))/
     &               (scpx(i,j)*scpy(i,j))
                dvdy=(v(i  ,j+1,k  )*scvx(i  ,j+1)-
     &                v(i  ,j  ,k  )*scvx(i  ,j  ))/
     &               (scpx(i,j)*scpy(i,j))
                if (dp(i,j,k).gt.0.0) wvelup(i,j,k)=wveldn(i,j,k-1)-0.5*
     &                ((u(i  ,j  ,k)-u(i  ,j  ,k-1))*dpdx(i  ,j  )+
     &                 (u(1  ,j  ,k)-u(1  ,j  ,k-1))*dpdx(1  ,j  )+
     &                 (v(i  ,j  ,k)-v(i  ,j  ,k-1))*dpdy(i  ,j  )+
     &                 (v(i  ,j+1,k)-v(i  ,j+1,k-1))*dpdy(i  ,j+1))
     &                       /onem
                wveldn(i,j,k)=wvelup(i,j,k  )+dp(i,j,k)*(dudx+dvdy)/onem
              endif
            endif
          enddo !j
c
        enddo !k
c
      endif !wvelfl
c
c --- set old velocity indices used for interpolation of float position
c --- perform full float update every two times this subroutine is called;
c --- at the intermediate times, just store old velocity fields for the next
c --- call
c
      if (mod(nstepfl,2).eq.1) then
        kold1 =kk
        kold2 =0
        go to 10
      else
        kold1 =0
        kold2 =kk
      endif
c
c --- turbulent horizontal velocity option
c
c --- to customize the parameter settings in blkdat.input that control
c --- the calculated turbulent velocity (tbvar, tdecri), refer to
c --- Griffa (1996), "Aplications of stochastic particles models to
c --- oceanographic problems" in "Stochatic Modelling in Physical
c --- Oceanography", pp. 114-140, Adler, Muller, and Rozovoskii, editors
c
      if (turbvel) then
c
c --- initialize random seeds and create random number table
c --- if iflag=1 (iflag=0), then time() is (is not) used to generate seeds
c
        iflag=1
        call system_clock(inisee)
        seed = 414957000-inisee
        iede = iseed1
        ied = iseed2
        if (iede .lt. 0) iede = iabs(iede)
        if (ied .lt. 0) ied = iabs(ied)
c
        call rantab_ini(rtab,iseed1,iseed2,iflag)
      endif
c
c ----------------------
c --- begin float loop
c ----------------------
c
      do nfl=1,nflt
c
c --- skip if float has previously run aground or exceeded termination time
        if(flt(nfl,1).le.-999.) then
          go to 22
        endif
c
c --- ier    = error flag for horizontal interpolation
c --- ntermn = float termination flag
c ---             -10 => reached termination time
c ---              -5 => exited domain
c ---              -1 => monte-carlo termination
c ---              >0 => ran aground
c --- ifladv = float horizontal advection flag
        ier=0
        ntermn=0
        ifladv=1
c
c --- do not advect floats on initial time step so that the initial
c --- positions and depths are written on the output file
        if (nstepfl.eq.0) then
          ifladv=0
        endif
c
c --- check if model time has reached float deployment time when it
c --- does, suppress float advection during first pass so the first
c --- output float location is exactly the input location
        if (timefl-flt(nfl,6).lt.-0.001) then
          go to 22
        endif
        if (kfloat(nfl).lt.0 .and. nstepfl.gt.1) then
          ifladv=0
        endif
c
c --- check if model time has reached float termination time
        if (flt(nfl,7).gt.0.0 .and.
     &      timefl-flt(nfl,7).gt. 0.001) then
          ntermn=-10
        endif
c
c --- monte-carlo determination of end-of-life
        if (dfmonc.gt.0) then
          cfmonc=0.685*2.0*dtturb/dfmonc  ! 0.685 - 50% reduction after dfmonc
          call rantab(rtab,iseed1,iseed2,numran)
          cfest=rtab(numran, 1)
          if (cfest.lt.cfmonc) then
            ntermn=-1
            go to 30
          endif
        endif
c
c ---------------------------------------------------------------------------
c --- search for the surrounding 16 grid points on the p, u, and v grids
c ---------------------------------------------------------------------------
c
        do j=1,jj-1
          do i=1,ii
c
c --- search for the surrounding p-grid points
            alomin=min(plonf(i  ,j),plonf(i  ,j+1),
     &                 plonf(i+1,j),plonf(i+1,j+1))
            alomax=max(plonf(i  ,j),plonf(i  ,j+1),
     &                 plonf(i+1,j),plonf(i+1,j+1))
            alamin=min(platf(i  ,j),platf(i  ,j+1),
     &                 platf(i+1,j),platf(i+1,j+1))
            alamax=max(platf(i  ,j),platf(i  ,j+1),
     &                 platf(i+1,j),platf(i+1,j+1))
c
c --- reference p grid point - determine if float has exited domain
            if (flt(nfl,1).ge.alomin.and.flt(nfl,1).lt.alomax.and.
     &          flt(nfl,2).ge.alamin.and.flt(nfl,2).lt.alamax) then
              ifltll(1)=i
              jfltll(1)=j
              if (((i.lt.2 .or. i.gt.idm-2) .and. .not.lperiod) .or.
     &             (j.lt.2 .or. j.gt.jdm-2)) then
                ntermn=-5
                go to 12
              endif
c
c --- search for the surrounding u and v grid points if they are on
c --- the staggered C-grid; otherwise use p grid points
c
              if (iuvgrd.eq.1) then  ! hycom c grid
c
c --- search for the surrounding u-grid points
                do i1=i-1,i+1
                  do j1=j-1,j+1
                    if (j1.lt.jdm) then
                      alomin=min(ulonf(i1  ,j1),ulonf(i1  ,j1+1),
     &                           ulonf(i1+1,j1),ulonf(i1+1,j1+1))
                      alomax=max(ulonf(i1  ,j1),ulonf(i1  ,j1+1),
     &                           ulonf(i1+1,j1),ulonf(i1+1,j1+1))
                      alamin=min(ulatf(i1  ,j1),ulatf(i1  ,j1+1),
     &                           ulatf(i1+1,j1),ulatf(i1+1,j1+1))
                      alamax=max(ulatf(i1  ,j1),ulatf(i1  ,j1+1),
     &                           ulatf(i1+1,j1),ulatf(i1+1,j1+1))
                    endif
c
c --- reference u grid point - determine if float has exited domain
                    if (flt(nfl,1).ge.alomin .and.
     &                  flt(nfl,1).lt.alomax .and.
     &                  flt(nfl,2).ge.alamin .and.
     &                  flt(nfl,2).lt.alamax) then
                      ifltll(2)=i1
                      jfltll(2)=j1
                      if (((i1.lt.2 .or. i1.gt.idm-2)
     &                     .and. .not.lperiod) .or.
     &                     (j1.lt.2 .or. j1.gt.jdm-2)) then
                        ntermn=-5
                        go to 12
                      endif
                      go to 11
                    endif
                  enddo !j1
                enddo !i1
   11           continue
c
c --- search for the surrounding v-grid points
                do i1=i-1,i+1
                  do j1=j-1,j+1
                    if (j1.lt.jdm) then
                      alomin=min(vlonf(i1  ,j1),vlonf(i1  ,j1+1),
     &                           vlonf(i1+1,j1),vlonf(i1+1,j1+1))
                      alomax=max(vlonf(i1  ,j1),vlonf(i1  ,j1+1),
     &                           vlonf(i1+1,j1),vlonf(i1+1,j1+1))
                      alamin=min(vlatf(i1  ,j1),vlatf(i1  ,j1+1),
     &                           vlatf(i1+1,j1),vlatf(i1+1,j1+1))
                      alamax=max(vlatf(i1  ,j1),vlatf(i1  ,j1+1),
     &                           vlatf(i1+1,j1),vlatf(i1+1,j1+1))
                    endif
c
c --- reference v grid point - determine if float has exited domain
                    if (flt(nfl,1).ge.alomin .and.
     &                  flt(nfl,1).lt.alomax .and.
     &                  flt(nfl,2).ge.alamin .and.
     &                  flt(nfl,2).lt.alamax) then
                      ifltll(3)=i1
                      jfltll(3)=j1
                      if (((i1.lt.2 .or. i1.gt.idm-2)
     &                     .and. .not.lperiod) .or.
     &                     (j1.lt.2 .or. j1.gt.jdm-2)) then
                        ntermn=-5
                      endif
                      go to 12
                    endif
                  enddo !j1
                enddo !i1
              else !non-hycom grid
c
c --- u and v on p grid points
                ifltll(2)=ifltll(1)
                jfltll(2)=jfltll(1)
                ifltll(3)=ifltll(1)
                jfltll(3)=jfltll(1)
              endif ! iuvgrd
            endif
          enddo !i
        enddo !j
c
 12     continue
c
        if     (nfl.eq.nfl_debug) then
          write(lp, 99) nfl,nflt,iar,nstepfl
          if (nonlatlon) then
            write(lp,100) 'model grid is curvilinear'
          else
            write(lp,100) 'model grid is rectangular'
          endif
          write(lp,101) nfl,ntermn,flt(nfl,1),flt(nfl,2),
     &                 (ifltll(i),jfltll(i),i=1,3),
     &                  plonf(ifltll(1),jfltll(1)),
     &                  platf(ifltll(1),jfltll(1))
  99      format(/'diagnostics for float',i6,' of',i6,', archive no.',
     &          i9/'float time step',i9)
 100      format(/,25a)
 101      format('float',i6,' ntermn:',i6,' position:',2(1pe12.4)/
     &           'lower left points, p,u,v:',3(2i5,2x)/
     &           '  plon,plat =',1pe12.4,1pe12.4)
          call flush(lp)
        endif !nfl_debug
c
c --- if float has exited domain or run aground as determined previously,
c --- jump ahead to the float termination code
        if (ntermn.eq.-5 .or. ntermn.eq.1) then
          go to 30
        endif
c
c --- set ptlon, ptlat for all horizontal interpolations from each grid
        ngrid=1
        do i1=1,4
          i=i1+ifltll(ngrid)-2
          do j1=1,4
            j=j1+jfltll(ngrid)-2
            ptlon(i1,j1,ngrid)=plonf(i,j)
            ptlat(i1,j1,ngrid)=platf(i,j)
          enddo
        enddo
c
        if (iuvgrd.eq.1) then
          ngrid=2
          do i1=1,4
            i=i1+ifltll(ngrid)-2
            do j1=1,4
              j=j1+jfltll(ngrid)-2
              ptlon(i1,j1,ngrid)=ulonf(i,j)
              ptlat(i1,j1,ngrid)=ulatf(i,j)
            enddo
          enddo
c
          ngrid=3
          do i1=1,4
            i=i1+ifltll(ngrid)-2
            do j1=1,4
              j=j1+jfltll(ngrid)-2
              ptlon(i1,j1,ngrid)=vlonf(i,j)
              ptlat(i1,j1,ngrid)=vlatf(i,j)
            enddo
          enddo
        else
          do i1=1,4
            do j1=1,4
              ptlon(i1,j1,2)=ptlon(i1,j1,1)
              ptlat(i1,j1,2)=ptlon(i1,j1,1)
              ptlon(i1,j1,3)=ptlon(i1,j1,1)
              ptlat(i1,j1,3)=ptlon(i1,j1,1)
            enddo
          enddo
        endif
c
c --- the float is assumed to remain within the same layer that it was in
c --- during the previous update unless it is the first advection time step
c --- for the float, in which case it is initially assumed to be in layer 1
c
        k=max(1,kfloat(nfl))
c
c --- mask p grid points that are on land or where the layer containing the
c --- float is a zero or nearly-zero thickness layer at the bottom
        ngrid=1
        ngood(ngrid)=0
        do i1=1,4
          i=mod(i1+ifltll(ngrid)-3,ii)+1
          do j1=1,4
            j=j1+jfltll(ngrid)-2
            if (depths(i,j).lt.0.01 .or. depths(i,j).eq.huge .or.
     &          depths(i,j)*onem-p(i,j,k).lt.tencm) then
              maskpt(i1,j1,ngrid)=.true.
            else
              maskpt(i1,j1,ngrid)=.false.
              ngood(ngrid)=ngood(ngrid)+1
            endif
            masktopo(i1,j1)=.false.
          enddo
        enddo
c
        if     (nfl.eq.nfl_debug) then
          write(lp,102) ngood(1)
 102      format('initial masking: no. of good p points',i4)
        endif !nfl_debug
c
c --------------------------------------------------
c --- determine the model layer containing the float
c --------------------------------------------------
c
c --- if this is not the first advection time step for the float, determine
c --- if the float has transited to another model layer
c
        if (kfloat(nfl).ge.1) then
c
c --- check if the float is deeper than the interpolated bottom
c --- depth - if so, it has run aground
          ngrid=1
          do i1=1,4
            i=mod(i1+ifltll(ngrid)-3,ii)+1
            do j1=1,4
              j=j1+jfltll(ngrid)-2
              varb2d(i1,j1)=p(i,j,kk+1)
            enddo
          enddo
          call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &                flt(nfl,1),flt(nfl,2),maskpt(1,1,ngrid),
     &                ngood(ngrid),ngrid,intpfl,radian,plo,ier)
          flt(nfl,3)=max(fldepm,flt(nfl,3))
c --- terminate float because it has run aground
          if (ier.eq.999) then
            ntermn=2
            go to 30
          endif
c
c --- has the float remained in the same layer?
          ngrid=1
          do i1=1,4
            i=mod(i1+ifltll(ngrid)-3,ii)+1
            do j1=1,4
              j=j1+jfltll(ngrid)-2
              varb2d(i1,j1)=p(i,j,k)
            enddo
          enddo
          call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &                flt(nfl,1),flt(nfl,2),maskpt(1,1,ngrid),
     &                ngood(ngrid),ngrid,intpfl,radian,phi,ier)
c
c --- terminate float because it has run aground
          if (ier.eq.999) then
            ntermn=2
            go to 30
          endif
c
          do i1=1,4
            i=mod(i1+ifltll(ngrid)-3,ii)+1
            do j1=1,4
              j=j1+jfltll(ngrid)-2
              varb2d(i1,j1)=p(i,j,k+1)
            enddo
          enddo
          call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &                flt(nfl,1),flt(nfl,2),maskpt(1,1,ngrid),
     &                ngood(ngrid),ngrid,intpfl,radian,plo,ier)
          phi=phi/onem
          plo=plo/onem
c
c --- if the float is in the same model layer, skip ahead
          if (flt(nfl,3).gt.phi .and. flt(nfl,3).le.plo) then
            kflt=kfloat(nfl)
            go to 40
          endif
c
c --- determine the new model layer containing the float
c
c --- did the float move up one layer?
          if (flt(nfl,3).le.phi .and. kfloat(nfl).gt.1) then
            k=kfloat(nfl)-1
            plo=phi
            do i1=1,4
              i=mod(i1+ifltll(ngrid)-3,ii)+1
              do j1=1,4
                j=j1+jfltll(ngrid)-2
                varb2d(i1,j1)=p(i,j,k)
              enddo
            enddo
            call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &                  flt(nfl,1),flt(nfl,2),maskpt(1,1,ngrid),
     &                  ngood(ngrid),ngrid,intpfl,radian,phi,ier)
            phi=phi/onem
            if (flt(nfl,3).gt.phi .and. flt(nfl,3).le.plo) then
              kflt=k
              go to 40
            endif
          endif
c
c --- did the float move down one layer?
          if (flt(nfl,3).gt.plo .and. kfloat(nfl).lt.kk) then
            k=kfloat(nfl)+1
            phi=plo
            do i1=1,4
              i=mod(i1+ifltll(ngrid)-3,ii)+1
              do j1=1,4
                j=j1+jfltll(ngrid)-2
                varb2d(i1,j1)=p(i,j,k)
              enddo
            enddo
            call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &                  flt(nfl,1),flt(nfl,2),maskpt(1,1,ngrid),
     &                  ngood(ngrid),ngrid,intpfl,radian,plo,ier)
            plo=plo/onem
            if (flt(nfl,3).gt.phi .and. flt(nfl,3).le.plo) then
              kflt=k
              go to 40
            endif
          endif
c
        endif
c
c --- if the float did not move up or down one layer, or if this is the
c --- first time step after float initialization, search for the layer
c --- containg the float from top to bottom
        ngrid=1
        plo=0.
        k2=0
        k3=0
        do k=1,kk
          k1=min(k+3,kk+1)
c
c --- if the depth of interface k1 is not deeper than the float at any
c --- surrounding grid points, skip to end of k-loop
          if (k2.eq.0) then
            do i1=1,4
              i=mod(i1+ifltll(ngrid)-3,ii)+1
              do j1=1,4
                j=j1+jfltll(ngrid)-2
                if (.not.maskpt(i1,j1,ngrid) .and.
     &              p(i,j,k1)/onem.gt.flt(nfl,3)) then
                  k2=1
                endif
              enddo
            enddo
          endif
          if (k2.eq.0) then
            go to 42
          endif
c
c --- mask points where the layer being tested rests on the bottom
          do i1=1,4
            i=mod(i1+ifltll(ngrid)-3,ii)+1
            do j1=1,4
              j=j1+jfltll(ngrid)-2
              if (k2.eq.1) then
                maskpi(i1,j1)=maskpt(i1,j1,ngrid)
              endif
              if (.not.maskpi(i1,j1) .and.
     &            depths(i,j)*onem-p(i,j,k).lt.tencm) then
                maskpi(i1,j1)=.true.
                ngood(ngrid)=ngood(ngrid)-1
              endif
            enddo
          enddo
          k2=2
c
c --- if too few good points remain, float has run aground
          if (ngood(ngrid).le.2) then
            ntermn=3
            go to 30
          endif
c
c --- interpolate to find plo
          phi=plo
          ngrid=1
          do i1=1,4
            i=mod(i1+ifltll(ngrid)-3,ii)+1
            do j1=1,4
              j=j1+jfltll(ngrid)-2
              varb2d(i1,j1)=p(i,j,k+1)
            enddo
          enddo
          call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &                flt(nfl,1),flt(nfl,2),maskpt(1,1,ngrid),
     &                ngood(ngrid),ngrid,intpfl,radian,plo,ier)
          plo=plo/onem
          if(plo-phi.gt.0.1) then
            k3=k
          else
            kflt=k3
            do i1=1,4
              do j1=1,4
                maskpt(i1,j1,ngrid)=maskpi(i1,j1)
              enddo
            enddo
            go to 40
          endif
          if (flt(nfl,3).lt.plo) then
            kflt=k
            do i1=1,4
              do j1=1,4
                maskpt(i1,j1,ngrid)=maskpi(i1,j1)
              enddo
            enddo
            go to 40
          endif
 42       continue
        enddo
c
c --- if layer selection fails, assume float has run aground
        print *,'warning - selection of float layer failed'
        ntermn=4
        go to 30
c
c --- we now know the model layer containing the float
 40     kfloat(nfl)=kflt
c
c ------------------------------------------
c --- set grid masks for u and v grid points
c ------------------------------------------
c
        k=kflt
        if (iuvgrd.eq.1) then  ! hycom c grid
          ngrid=2
          ngood(ngrid)=0
          do i1=1,4
            i=mod(i1+ifltll(ngrid)-3,ii)+1
            do j1=1,4
              j=j1+jfltll(ngrid)-2
              if (depthu(i,j).lt.onecm .or. depthu(i,j).eq.huge .or.
     &            depthu(i,j)-pu(i,j,k).lt.tencm) then
                maskpt(i1,j1,ngrid)=.true.
              else
                maskpt(i1,j1,ngrid)=.false.
                ngood(ngrid)=ngood(ngrid)+1
              endif
            enddo
          enddo
c
          ngrid=3
          ngood(ngrid)=0
          do i1=1,4
            i=mod(i1+ifltll(ngrid)-3,ii)+1
            do j1=1,4
              j=j1+jfltll(ngrid)-2
              if (depthv(i,j).lt.onecm .or. depthv(i,j).eq.huge .or.
     &            depthv(i,j)-pv(i,j,k).lt.tencm) then
                maskpt(i1,j1,ngrid)=.true.
              else
                maskpt(i1,j1,ngrid)=.false.
                ngood(ngrid)=ngood(ngrid)+1
              endif
            enddo
          enddo
        else ! non-hycom grid
          do i1=1,4
            do j1=1,4
              maskpt(i1,j1,2)=maskpt(i1,j1,1)
              maskpt(i1,j1,3)=maskpt(i1,j1,1)
              ngood(2)=ngood(1)
              ngood(3)=ngood(1)
            enddo
          enddo
        endif !iuvgrd
c
        if     (nfl.eq.nfl_debug) then
          write(lp,103) nfl,k,phi,flt(nfl,3),plo,q
          write(lp,104) ngood
 103      format('nfl,k,phi,pflt,plo,q',i6,i3,1p,4e12.4)
 104      format('number of good points, p,u,v',3i4)
        endif !nfl_debug
c
c --- if too few good p, u, v grid points, assume float has run aground
        if (ngood(1).le.2 .or. ngood(2).le.2 .or. ngood(3).le.2) then
          ntermn=5
          go to 30
        endif
c
c --- set vertical indices, and also set q for vertical interpolation
        k=kfloat(nfl)
        k1=max(1,k-1)
        k2=min(kk,k+1)
        q=(plo-flt(nfl,3))/max(0.001,plo-phi)
c
c ----------------------------------------------------------
c --- advect the float horizontally, then move it vertically
c --- do not move float if it is a mooring instrument
c ----------------------------------------------------------
c
c ----------------------------------
c --- horizontal advection of floats
c ----------------------------------
c
c --- interpolate u,v to float location - velocities at the new time and
c --- at two earlier times are used for the runga-kutta time interpolation
c
        xpos0=flt(nfl,1)
        ypos0=flt(nfl,2)
c
        ngrid=2
        do i1=1,4
          i=mod(i1+ifltll(ngrid)-3,ii)+1
          do j1=1,4
            j=j1+jfltll(ngrid)-2
            varb2d(i1,j1)=dlondx(i,j)*uold2(i,j,k+kold2)
          enddo
        enddo
        call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &              xpos0,ypos0,maskpt(1,1,ngrid),
     &              ngood(ngrid),ngrid,intpfl,radian,uflt1,ier)
c
c --- terminate float because it has run aground
        if (ier.eq.999) then
          ntermn=6
          go to 30
        endif
c
        if (nonlatlon) then
          ngrid=3
          do i1=1,4
            i=mod(i1+ifltll(ngrid)-3,ii)+1
            do j1=1,4
              j=j1+jfltll(ngrid)-2
              varb2d(i1,j1)=dlondy(i,j)*vold2(i,j,k+kold2)
            enddo
          enddo
          call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &                xpos0,ypos0,maskpt(1,1,ngrid),
     &                ngood(ngrid),ngrid,intpfl,radian,ufltm,ier)
          uflt1=uflt1+ufltm
c
c --- terminate float because it has run aground
          if (ier.eq.999) then
            ntermn=7
            go to 30
          endif
        endif
c
        ngrid=3
        do i1=1,4
          i=mod(i1+ifltll(ngrid)-3,ii)+1
          do j1=1,4
            j=j1+jfltll(ngrid)-2
            varb2d(i1,j1)=dlatdy(i,j)*vold2(i,j,k+kold2)
          enddo
        enddo
        call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &              xpos0,ypos0,maskpt(1,1,ngrid),
     &              ngood(ngrid),ngrid,intpfl,radian,vflt1,ier)
c
c --- terminate float because it has run aground
        if (ier.eq.999) then
          ntermn=8
          go to 30
        endif
c
        if (nonlatlon) then
          ngrid=2
          do i1=1,4
            i=mod(i1+ifltll(ngrid)-3,ii)+1
            do j1=1,4
              j=j1+jfltll(ngrid)-2
              varb2d(i1,j1)=dlatdx(i,j)*uold2(i,j,k+kold2)
            enddo
          enddo
          call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &                xpos0,ypos0,maskpt(1,1,ngrid),
     &                ngood(ngrid),ngrid,intpfl,radian,vfltm,ier)
          vflt1=vflt1+vfltm
c
c --- terminate float because it has run aground
          if (ier.eq.999) then
            ntermn=9
            go to 30
          endif
        endif
c
        xpos1=flt(nfl,1)+deltfl*uflt1
        ypos1=flt(nfl,2)+deltfl*vflt1
c
        ngrid=2
        do i1=1,4
          i=mod(i1+ifltll(ngrid)-3,ii)+1
          do j1=1,4
            j=j1+jfltll(ngrid)-2
            varb2d(i1,j1)=dlondx(i,j)*uold2(i,j,k+kold1)
          enddo
        enddo
        call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &              xpos1,ypos1,maskpt(1,1,ngrid),
     &              ngood(ngrid),ngrid,intpfl,radian,uflt2,ier)
        if (ier.eq.999) then
          uflt2=uflt1
        elseif (nonlatlon) then
c
          ngrid=3
          do i1=1,4
            i=mod(i1+ifltll(ngrid)-3,ii)+1
            do j1=1,4
              j=j1+jfltll(ngrid)-2
              varb2d(i1,j1)=dlondy(i,j)*vold2(i,j,k+kold1)
            enddo
          enddo
          call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &                xpos1,ypos1,maskpt(1,1,ngrid),
     &                ngood(ngrid),ngrid,intpfl,radian,ufltm,ier2)
          if (ier2.eq.999) then
            uflt2=uflt1
          else
            uflt2=uflt2+ufltm
          endif
        endif
c
        ngrid=3
        do i1=1,4
          i=mod(i1+ifltll(ngrid)-3,ii)+1
          do j1=1,4
            j=j1+jfltll(ngrid)-2
            varb2d(i1,j1)=dlatdy(i,j)*vold2(i,j,k+kold1)
          enddo
        enddo
        call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &              xpos1,ypos1,maskpt(1,1,ngrid),
     &              ngood(ngrid),ngrid,intpfl,radian,vflt2,ier)
        if (ier.eq.999) then
          vflt2=vflt1
        elseif (nonlatlon) then
c
          ngrid=2
          do i1=1,4
            i=mod(i1+ifltll(ngrid)-3,ii)+1
            do j1=1,4
              j=j1+jfltll(ngrid)-2
              varb2d(i1,j1)=dlatdx(i,j)*uold2(i,j,k+kold1)
            enddo
          enddo
          call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &                xpos1,ypos1,maskpt(1,1,ngrid),
     &                ngood(ngrid),ngrid,intpfl,radian,vfltm,ier2)
          if (ier2.eq.999) then
            vflt2=vflt1
          else
            vflt2=vflt2+vfltm
          endif
        endif
c
        xpos2=flt(nfl,1)+deltfl*uflt2
        ypos2=flt(nfl,2)+deltfl*vflt2
        ngrid=2
        do i1=1,4
          i=mod(i1+ifltll(ngrid)-3,ii)+1
          do j1=1,4
            j=j1+jfltll(ngrid)-2
            varb2d(i1,j1)=dlondx(i,j)*uold2(i,j,k+kold1)
          enddo
        enddo
        call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &              xpos2,ypos2,maskpt(1,1,ngrid),
     &              ngood(ngrid),ngrid,intpfl,radian,uflt3,ier)
        if (ier.eq.999) then
          uflt3=uflt1
        elseif (nonlatlon) then
c
          ngrid=3
          do i1=1,4
            i=mod(i1+ifltll(ngrid)-3,ii)+1
            do j1=1,4
              j=j1+jfltll(ngrid)-2
              varb2d(i1,j1)=dlondy(i,j)*vold2(i,j,k+kold1)
            enddo
          enddo
          call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &                xpos2,ypos2,maskpt(1,1,ngrid),
     &                ngood(ngrid),ngrid,intpfl,radian,ufltm,ier2)
          if (ier2.eq.999) then
            uflt3=uflt1
          else
            uflt3=uflt3+ufltm
          endif
        endif
c
        ngrid=3
        do i1=1,4
          i=mod(i1+ifltll(ngrid)-3,ii)+1
          do j1=1,4
            j=j1+jfltll(ngrid)-2
            varb2d(i1,j1)=dlatdy(i,j)*vold2(i,j,k+kold1)
          enddo
        enddo
        call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &              xpos2,ypos2,maskpt(1,1,ngrid),
     &              ngood(ngrid),ngrid,intpfl,radian,vflt3,ier)
        if (ier.eq.999) then
          vflt3=vflt1
        elseif (nonlatlon) then
c
          ngrid=2
          do i1=1,4
            i=mod(i1+ifltll(ngrid)-3,ii)+1
            do j1=1,4
              j=j1+jfltll(ngrid)-2
              varb2d(i1,j1)=dlatdx(i,j)*uold2(i,j,k+kold1)
            enddo
          enddo
          call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &                xpos2,ypos2,maskpt(1,1,ngrid),
     &                ngood(ngrid),ngrid,intpfl,radian,vfltm,ier2)
          if (ier2.eq.999) then
            vflt3=vflt1
          else
            vflt3=vflt3+vfltm
          endif
        endif
c
        xpos3=flt(nfl,1)+2.0*deltfl*uflt3
        ypos3=flt(nfl,2)+2.0*deltfl*vflt3
c
        ngrid=2
        do i1=1,4
          i=mod(i1+ifltll(ngrid)-3,ii)+1
          do j1=1,4
            j=j1+jfltll(ngrid)-2
            varb2d(i1,j1)=dlondx(i,j)*u(i,j,k)
          enddo
        enddo
        call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &              xpos3,ypos3,maskpt(1,1,ngrid),
     &              ngood(ngrid),ngrid,intpfl,radian,uflt4,ier)
        if (ier.eq.999) then
          ntermn=9
        elseif (ier.eq.0 .and. nonlatlon) then
c
          ngrid=3
          do i1=1,4
            i=mod(i1+ifltll(ngrid)-3,ii)+1
            do j1=1,4
              j=j1+jfltll(ngrid)-2
              varb2d(i1,j1)=dlondy(i,j)*v(i,j,k)
            enddo
          enddo
          call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &                xpos3,ypos3,maskpt(1,1,ngrid),
     &                ngood(ngrid),ngrid,intpfl,radian,ufltm,ier2)
          if (ier2.eq.999) then
            uflt4=uflt2+uflt3-uflt1
          else
            uflt4=uflt4+ufltm
          endif
        endif
c
        ngrid=3
        do i1=1,4
          i=mod(i1+ifltll(ngrid)-3,ii)+1
          do j1=1,4
            j=j1+jfltll(ngrid)-2
            varb2d(i1,j1)=dlatdy(i,j)*v(i,j,k)
          enddo
        enddo
        call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &              xpos3,ypos3,maskpt(1,1,ngrid),
     &              ngood(ngrid),ngrid,intpfl,radian,vflt4,ier)
        if (ier.eq.999) then
          ntermn=9
        elseif (ier.eq.0 .and. nonlatlon) then
c
          ngrid=2
          do i1=1,4
            i=mod(i1+ifltll(ngrid)-3,ii)+1
            do j1=1,4
              j=j1+jfltll(ngrid)-2
              varb2d(i1,j1)=dlatdx(i,j)*u(i,j,k)
            enddo
          enddo
          call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &                xpos3,ypos3,maskpt(1,1,ngrid),
     &                ngood(ngrid),ngrid,intpfl,radian,vfltm,ier2)
          if (ier2.eq.999) then
            vflt4=vflt2+vflt3-vflt1
          else
            vflt4=vflt4+vfltm
          endif
        endif
c
        uflt=(uflt1+2.0*uflt2+2.0*uflt3+uflt4)/6.0
        vflt=(vflt1+2.0*vflt2+2.0*vflt3+vflt4)/6.0
c
c --- add turbulent velocity to u and v if requested
c
        if (turbvel) then
          call rantab(rtab,iseed1,iseed2,numran)
          uturb=uturb0*rtab(numran, 2)
          call rantab(rtab,iseed1,iseed2,numran)
          vturb=uturb0*rtab(numran, 2)
c
c --- convert uturb, vturb to d(longitude)/dt, d(latitude)/dt
c --- interpolate dlondx, dlondy, dlatdx, dlatdy to the float location
c
          ngrid=2
          do i1=1,4
            i=mod(i1+ifltll(ngrid)-3,ii)+1
            do j1=1,4
              j=j1+jfltll(ngrid)-2
              varb2d(i1,j1)=dlondx(i,j)
            enddo
          enddo
          call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &                xpos0,ypos0,maskpt(1,1,ngrid),
     &                ngood(ngrid),ngrid,intpfl,radian,dlodx,ier)
          if (nonlatlon) then
            do i1=1,4
              i=mod(i1+ifltll(ngrid)-3,ii)+1
              do j1=1,4
                j=j1+jfltll(ngrid)-2
                varb2d(i1,j1)=dlatdx(i,j)
              enddo
            enddo
            call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &                  xpos0,ypos0,maskpt(1,1,ngrid),
     &                  ngood(ngrid),ngrid,intpfl,radian,dladx,ier)
            ngrid=3
            do i1=1,4
              i=mod(i1+ifltll(ngrid)-3,ii)+1
              do j1=1,4
                j=j1+jfltll(ngrid)-2
                 varb2d(i1,j1)=dlondy(i,j)
              enddo
            enddo
            call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &                  xpos0,ypos0,maskpt(1,1,ngrid),
     &                  ngood(ngrid),ngrid,intpfl,radian,dlody,ier)
          else
            dladx=0.0
            dlody=0.0
          endif
          ngrid=3
          do i1=1,4
            i=mod(i1+ifltll(ngrid)-3,ii)+1
            do j1=1,4
              j=j1+jfltll(ngrid)-2
              varb2d(i1,j1)=dlatdy(i,j)
            enddo
          enddo
          call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &                xpos0,ypos0,maskpt(1,1,ngrid),
     &                ngood(ngrid),ngrid,intpfl,radian,dlady,ier)
c
c --- add turbulent velocity to uflt, vflt
c
          uturb1=uturb*dlodx+vturb*dlody
          vturb1=uturb*dladx+vturb*dlady
c
      if     (nfl.eq.nfl_debug) then
        write(lp,105) nfl,uflt,uturb1,vflt,vturb1
 105    format('nfl,uflt,uturb1,vflt,vturb1',i5,1p,4e12.4)
      endif !nfl_debug
c
          uflt=(1.0-dtturb*tdecri)*uflt+uturb1
          vflt=(1.0-dtturb*tdecri)*vflt+vturb1
c
        endif !turbvel
c
c --- update float horizontal position
c
        if (ifladv.eq.1) then
          flt(nfl,1)=flt(nfl,1)+2.0*deltfl*uflt
          flt(nfl,2)=flt(nfl,2)+2.0*deltfl*vflt
        endif
c
c --- check if float has left domain
c
c --- if global periodic domain, make sure that the longitudes
c --- remain within the original 360-degree boundaries
c
        alomin=minval(plon(:,jfltll(1)))
        alomax=maxval(plon(:,jfltll(1)))
        alamin=minval(plat(ifltll(1),:))
        alamax=maxval(plat(ifltll(1),:))
        if ((flt(nfl,2).le.alamin) .or.
     &      (flt(nfl,2).ge.alamax)) then
          ntermn=-5
          go to 30
        endif
        if (lperiod) then
          if     (flt(nfl,1).lt.alomin) then
            flt(nfl,1)=flt(nfl,1)+360.0
          elseif (flt(nfl,1).gt.alomax) then
            flt(nfl,1)=flt(nfl,1)-360.0
          endif
        else
          if ((flt(nfl,1).le.alomin) .or.
     &        (flt(nfl,1).ge.alomax)) then
            ntermn=-5
            go to 30
        endif
c
      endif
c
      if     (nfl.eq.nfl_debug) then
        write(lp,106) nfl,flt(nfl,1),xpos0,xpos1,xpos2,xpos3,
     &                uflt1,uflt2,uflt3,uflt4,2.0*deltfl*uflt
        write(lp,107) nfl,flt(nfl,2),ypos0,ypos1,ypos2,ypos3,
     &                vflt1,vflt2,vflt3,vflt4,2.0*deltfl*vflt
        write(lp,108) nfl,uflt,vflt,flt(nfl,1),flt(nfl,2)
 106    format('nfl,x-position',i6,1p,5e12.4/'    u-velocity',6x,5e12.4)
 107    format('nfl,y-position',i6,1p,5e12.4/'    v-velocity',6x,5e12.4)
 108    format('nfl,udeg,vdeg',i6,1p,2e13.5/'new position',2e13.5)
      endif !nfl_debug
c
c -----------------------------------------------
c --- vertical advection of 3-d lagrangian floats
c -----------------------------------------------
c
c --- horizontally interpolate diagnosed vertical velocity to the float
c --- location at the new time and at two earlier times to execute the
c --- runga-kutta time interpolation
c
        if (wvelfl .and. ifltyp(nfl).eq.1) then
c
          ngrid=1
          do i1=1,4
            i=mod(i1+ifltll(ngrid)-3,ii)+1
            do j1=1,4
              j=j1+jfltll(ngrid)-2
              wvhi=wold2u(i,j,k+kold2)
              wvlo=wold2d(i,j,k+kold2)
              varb2d(i1,j1)=q*wvhi+(1.0-q)*wvlo
            enddo
          enddo
          call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &                xpos0,ypos0,maskpt(1,1,ngrid),
     &                ngood(ngrid),ngrid,intpfl,radian,wflt1,ier)
c
c --- terminate float because it has run aground
            if (ier.eq.999) then
              ntermn=10
              go to 30
            endif
c
          ngrid=1
          do i1=1,4
            i=mod(i1+ifltll(ngrid)-3,ii)+1
            do j1=1,4
              j=j1+jfltll(ngrid)-2
              wvhi=wold2u(i,j,k+kold1)
              wvlo=wold2d(i,j,k+kold1)
              varb2d(i1,j1)=q*wvhi+(1.0-q)*wvlo
            enddo
          enddo
          call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &                xpos1,ypos1,maskpt(1,1,ngrid),
     &                ngood(ngrid),ngrid,intpfl,radian,wflt2,ier)
          if (ier.eq.999) then
            wflt2=wflt1
          endif
c
          call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &                xpos2,ypos2,maskpt(1,1,ngrid),
     &                ngood(ngrid),ngrid,intpfl,radian,wflt3,ier)
          if (ier.eq.999) then
            wflt3=wflt1
          endif
c
        endif
c
        if (wvelfl) then
          ngrid=1
          do i1=1,4
            i=mod(i1+ifltll(ngrid)-3,ii)+1
            do j1=1,4
              j=j1+jfltll(ngrid)-2
              wvhi=wvelup(i,j,k)
              wvlo=wveldn(i,j,k)
              varb2d(i1,j1)=q*wvhi+(1.0-q)*wvlo
            enddo
          enddo
          call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &                xpos3,ypos3,maskpt(1,1,ngrid),
     &                ngood(ngrid),ngrid,intpfl,radian,wflt4,ier)
          if (ier.eq.999) then
            wflt4=wflt2+wflt3-wflt1
          endif
        endif
c
        if (ifltyp(nfl).eq.1) then
c
          wflt=deltfl*(wflt1+2.0*wflt2+2.0*wflt3+wflt4)/3.0
          if (ifladv.eq.1) then
            flt(nfl,3)=max(fldepm,flt(nfl,3)-wflt)
          endif
          if     (nfl.eq.nfl_debug) then
            write(lp,109) nfl,flt(nfl,3),
     &                    wflt1,wflt2,wflt3,wflt4,wflt
 109        format('nfl,w',i6,1p,5e13.5/11x,e13.5)
          endif !nfl_debug
c
        elseif (ifltyp(nfl).eq.2) then
c
c ---------------------------------------
c --- vertical motion of isopycnic floats
c ---------------------------------------
c
c --- set float depth to the vertically interpolated depth of the specified
c --- density interface
c
c --- assume float runs aground if no water denser than the specified density
c --- exists in the water column
c
c --- limit float depth to fldepm if no water lighter than the specified
c --- density exists in the water column
c
c --- find smallest k where float density is exceeded by the density in
c --- layer k at at least one of the good 16 surrounding grid points
          ngrid=1
          ngoodi=ngood(ngrid)
          do k=1,kk
            do i1=1,4
              i=mod(i1+ifltll(ngrid)-3,ii)+1
              do j1=1,4
                j=j1+jfltll(ngrid)-2
                maskpi(i1,j1)=maskpt(i1,j1,ngrid)
                if (.not.maskpi(i1,j1)) then
                  if (th3d(i,j,k).gt.flt(nfl,8)) then
                    kflt=k
                    go to 33
                  endif
               endif
              enddo
            enddo
          enddo
   33     continue
c
c --- move downward from layer kflt to find the two layers with densities
c --- bracketing the assigned float density
          do i1=1,4
            i=mod(i1+ifltll(ngrid)-3,ii)+1
            do j1=1,4
              j=j1+jfltll(ngrid)-2
              if (.not.maskpi(i1,j1)) then
                if (kflt.eq.1) then
                  phi=0.
                  plo=.5*dp(i,j,kflt)/onem
                  if(flt(nfl,8).le.th3d(i,j,kflt)) then
                    varb2d(i1,j1)=max(fldepm,plo)
                    go to 50
                  endif
                endif
                do k=max(2,kflt),kk
                  phi=p(i,j,k-1)/onem
                  plo=p(i,j,k  )/onem
                  thhi=th3d(i,j,k-1)
                  thlo=th3d(i,j,k  )
                  if(flt(nfl,8).gt.thhi .and. flt(nfl,8).le.thlo) then
                    qisop=max(0.0,min(1.0,(thlo-flt(nfl,8))/
     &                    max(epsil,thlo-thhi)))
                    varb2d(i1,j1)=max(fldepm,qisop*phi+(1.-qisop)*plo)
                    if (depths(i,j)-varb2d(i1,j1).lt.0.1) then
                      maskpi(i1,j1)=.true.
                      ngoodi=ngoodi-1
                    endif
                    go to 50
                  endif
                enddo
 50             continue
              endif
            enddo
          enddo
          if (ngoodi.le.7) then
            ntermn=11
            go to 30
          endif
          call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &                flt(nfl,1),flt(nfl,2),maskpi,
     &                ngoodi,ngrid,intpfl,radian,depiso,ier)
          if (ier.eq.999) then
            ntermn=12
            go to 30
          endif
          flt(nfl,3)=max(fldepm,depiso)
          if     (nfl.eq.nfl_debug) then
            write(lp,110) nfl,flt(nfl,3)
 110        format('nfl,new isopycnic depth',i6,1p,e13.5)
          endif !nfl_debug
c
        endif
c
c --- recalculate q. assume float does not leave the layer that it was
c --- originally diagnosed to be within.
c
            q=max(0.0,min(1.0,(plo-flt(nfl,3))/max(0.001,plo-phi)))
c
c ---   water depth interpolation
            if (ifltyp(nfl).ne.4 .or. flt(nfl,9).eq.0.0) then
              ngrid=1
              do i1=1,4
                i=i1+ifltll(ngrid)-2
                do j1=1,4
                  j=j1+jfltll(ngrid)-2
                  varb2d(i1,j1)=max(0.0,min(8000.0,depths(i,j)))
                enddo
              enddo
              call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &                    flt(nfl,1),flt(nfl,2),masktopo(1,1),
     &                    16,ngrid,intpfl,radian,depflt,ier)
              if (depflt.lt.flt(nfl,3)) then
                ntermn=13
                go to 30
              endif
              flt(nfl,9)=depflt
            endif
c
c --- temperature interpolation
            ngrid=1
            do i1=1,4
              i=i1+ifltll(ngrid)-2
              do j1=1,4
                j=j1+jfltll(ngrid)-2
                k1=max(1 ,k-1)
                k2=min(kk,k+1)
                varb2d(i1,j1)=
     &              0.5*(     q *(temp(i,j,k1)+temp(i,j,k))+
     &                   (1.0-q)*(temp(i,j,k2)+temp(i,j,k)))
              enddo
            enddo
            call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &                  flt(nfl,1),flt(nfl,2),maskpt(1,1,ngrid),
     &                  ngood(ngrid),ngrid,intpfl,radian,tflt,ier)
            flt(nfl,4)=tflt
c
c --- salinity interpolation (only implemented for hycom archives)
            if (iuvgrd.eq.1) then
              ngrid=1
              do i1=1,4
                i=i1+ifltll(ngrid)-2
                do j1=1,4
                  j=j1+jfltll(ngrid)-2
                  k1=max(1 ,k-1)
                  k2=min(kk,k+1)
                  varb2d(i1,j1)=
     &                0.5*(     q *(saln(i,j,k1)+saln(i,j,k))+
     &                     (1.0-q)*(saln(i,j,k2)+saln(i,j,k)))
                enddo
              enddo
              call intrph(varb2d,ptlon(1,1,ngrid),ptlat(1,1,ngrid),
     &                    flt(nfl,1),flt(nfl,2),maskpt(1,1,ngrid),
     &                    ngood(ngrid),ngrid,intpfl,radian,sflt,ier)
              flt(nfl,5)=sflt
            endif
c
c ----------------------------------------------------
c --- flag floats that have run aground or left domain
c ----------------------------------------------------
c
c --- jump ahead if the float has not reached its termination time
        if (ntermn.gt.-10) go to 20
c
c --- the following code block is entered when the previous code has
c --- detected that the float needs to be terminated
 30     flt(nfl,1)=-999.
        flt(nfl,2)=-999.
        if (ntermn.gt.0) then
          write(lp,112) nfl,iar,nstepfl,ier,ntermn,ngood
 112      format('float',i6,' aground, time step',i9,
     &           ' float time step',i9/'   ier ',i3,' exit point',
     &            i3,' ngood 1-4',4i3)
        elseif (ntermn.eq.-5) then
          write(lp,113) nfl
 113      format('float',i6,' exits domain')
        elseif (ntermn.eq.-10) then
          write(lp,114) nfl
 114      format('float',i6,' reaches termination time')
        elseif (ntermn.eq.-1) then
          write(lp,115) nfl
 115      format('float',i6,' evaporates or otherwise disappears')
        endif
c
 20     continue
c
 22     continue
c
c ---------------------
c --- end of float loop
c ---------------------
c
      enddo !nfl
c
c -----------------------------------
c --- output float array for analysis
c -----------------------------------
c
      write(lp,*) 'storing float parameters'
      open(unit=nt3,file=flnm_fo,status='unknown',
     &     form='formatted',position='append')
      do nfl=1,nflt
        if (flt(nfl,1).gt.-999. .and. timefl-flt(nfl,6).ge.-0.001)
     &      then
          if (iuvgrd.eq.1) then
            write(nt3,901) iflnum(nfl),timefl,kfloat(nfl),
     &                    (flt(nfl,j),j=1,5),flt(nfl,9)
          else   ! salinity not implemented for iuvgrd=0
            write(nt3,902) iflnum(nfl),timefl,kfloat(nfl),
     &                    (flt(nfl,j),j=1,4),flt(nfl,9)
          endif
 901      format(i9,f11.4,i3,2f10.4,f11.4,2f9.4,f11.4)
 902      format(i9,f11.4,i3,2f10.4,f11.4,f9.4,f11.4)
        endif
      enddo
      close(nt3)
c
c ----------------------------------------------------
c --- store old velocities for next time interpolation
c ----------------------------------------------------
c
 10   continue
c
      do j=1,jj
c
        do i=1,ii
          if (iu(i,j).eq.1) then
            do k=1,kk
              uold2(i,j,k+kold2)=u(i,j,k)
            enddo
          endif
c
          if (iv(i,j).eq.1) then
            do k=1,kk
              vold2(i,j,k+kold2)=v(i,j,k)
            enddo
          endif
c          
          if (ip(i,j).eq.1) then
            do k=1,kk
              wold2u(i,j,k+kold2)=wvelup(i,j,k)
              wold2d(i,j,k+kold2)=wveldn(i,j,k)
            enddo
          endif
        enddo
c
      enddo !j
c
      return
      end subroutine floats_advect

      subroutine intrph(varb2d,ptlon,ptlat,fllon,fllat,maskpt,
     &                  ngood,ngrid,intpfl,radian,vrbint,ier)
c
c ----------------------------------------------------
c --- 2-d polynomial or nearest neighbor interpolation
c ----------------------------------------------------
c
c --- polynomial interpolation code is adapted from the parameter matrix
c --- objective analysis algorithm of mariano and brown (1994), specifically
c --- the code used to calculate the large scale 2-d trend surface
c
      implicit none
      real varb2d(4,4),ptlon(4,4),ptlat(4,4)
      real fllon,fllat,fllon2,fllat2,vfbint
      real wghtx,wghty,vrbint,suma,wghts,xcen,ycen,xo,yo,xsd,ysd,var
      real dx,dy,dxflt,dyflt,det,denom,radian
      real at(16),xt(16),yt(16),parm(12),rtr(10,10),rtri(10,10),rta(10)
      integer iarea(200)
      integer ngood,ngrid,intpfl,ier,iflag,ict,nptmin
      integer i,j,nd,nds,np,na
      logical maskpt(4,4)
c
c --- nptmin is the minimum number of points required for 2-d interpolation
c --- to be performed - otherwise, nearest neighbor interpolation is used
c
      data nptmin /10/
c
c --- na is the row dimension of rtr
      data na /10/
c
c --- set the 1-d arrays of field values and horizontal position
c --- find and remove central latitude and longitude of the good points
c --- correct horizontal position for earth's curvature to lowest order
c
      ict=0
      xcen=0.0
      ycen=0.0
      do i=1,4
        do j=1,4
          if (.not.maskpt(i,j)) then
            ict=ict+1
            at(ict)=varb2d(i,j)
            xt(ict)=ptlon(i,j)
            xcen=xcen+xt(ict)
            yt(ict)=ptlat(i,j)
            ycen=ycen+yt(ict)
          endif
        enddo
      enddo
c
      np=ict
c
c --- assume float has run aground if <=4 water points
c
      if(np.le.2) then
        vrbint=0.0
        ier=999
        return
      endif
c
c --- set interpolated value to at(1) if all at array values are identical
      iflag=0
      do ict=2,np
        if(at(ict).ne.at(1)) iflag=1
      enddo
      if(iflag.eq.0) then
        vrbint=at(1)
        return
      endif
c
      xcen=xcen/np
      ycen=ycen/np
      denom=cos(ycen*radian)
c
      do ict=1,np
        xt(ict)=(xt(ict)-xcen)*cos(yt(ict)*radian)/denom
        yt(ict)=(yt(ict)-ycen)*cos(0.5*(yt(ict)+ycen)*radian)/denom
c
cdiag   write(6,101) ngrid,ngood,ict,at(ict),xt(ict),yt(ict)
 101    format('poly. - ngrid,ngood,ict,at,xt,yt:'
     &       /10x,3i4,1p,3e13.5)
c
      enddo
c
      fllon2=(fllon-xcen)*cos(fllat*radian)/denom
      fllat2=(fllat-ycen)*cos(0.5*(fllat+ycen)*radian)/denom
c
c --- perform the interpolation
c
      ier=0
c
      if (intpfl.eq.0 .and. ngood.ge.nptmin) then
c
c --- 2-d, second order polynomial interpolation
c
c --- set parameters for quadratic polynomial and zero the parm array
        nds=2
        nd=6
        do i=1,12
          parm(i)=0.0
        enddo
c
c --- calculate xo, the mean of xt, and yo, the mean of yt and their
c --- standard deviations from np observations 
c
        call f_stat(xt,np,xo,var,xsd)
        call f_stat(yt,np,yo,var,ysd)
c   
        do i=1,nd
          rta(i)=0.0
          do j=1,nd
            rtr(i,j)=0.0
          enddo
        enddo
c
c --- the next part of the code is a little messy, but we save alot of
c --- disk space by doing it this way
c
        do 20 ict=1,np
          dx=xt(ict)-xo
          dy=yt(ict)-yo
          rtr(1,2)=rtr(1,2) + dx
          rtr(1,3)=rtr(1,3) + dy
          rtr(2,2)=rtr(2,2) + dx**2
          rtr(2,3)=rtr(2,3) + dx*dy
          rtr(3,3)=rtr(3,3) + dy**2
          rta(1)=rta(1) + at(ict)
          rta(2)=rta(2) + dx*at(ict)
          rta(3)=rta(3) + dy*at(ict)
          if(nds.eq.1) go to 20
          rtr(2,4)=rtr(2,4) + (dx**3)/2.
          rtr(3,5)=rtr(3,5) + (dy**3)/2.
          rtr(4,4)=rtr(4,4) + (dx**4)/4.
          rtr(5,5)=rtr(5,5) + (dy**4)/4.
          rtr(3,4)=rtr(3,4) + (dx**2)*dy/2.
          rtr(2,5)=rtr(2,5) + (dy**2)*dx/2.
          rtr(4,6)=rtr(4,6) + (dx**3)*dy/2.
          rtr(5,6)=rtr(5,6) + (dy**3)*dx/2.
          rtr(6,6)=rtr(6,6) + (dx**2)*(dy**2)
          rta(4)=rta(4) + (dx**2)*at(ict)/2.
          rta(5)=rta(5) + (dy**2)*at(ict)/2.
          rta(6)=rta(6) + dx*dy*at(ict)
          if(nds.ne.3) go to 20
          rtr(4,7)=rtr(4,7) + (dx**5)/12.
          rtr(4,8)=rtr(4,8) + (dx**2)*(dy**3)/12.
          rtr(4,9)=rtr(4,9) + (dx**4)*dy/4.
          rtr(4,10)=rtr(4,10) + (dx**3)*(dy**2)/4.
          rtr(5,8)=rtr(5,8) + (dy**5)/12.
          rtr(5,10)=rtr(5,10) + dx*(dy**4)/4.
          rtr(6,7)=rtr(6,7) + dy*(dx**4)/6.
          rtr(7,7)=rtr(7,7) + (dx**6)/36.
          rtr(7,8)=rtr(7,8) + (dx**3)*(dy**3)/36.
          rtr(7,9)=rtr(7,9) + (dx**5)*dy/12.
          rtr(7,10)=rtr(7,10) + (dx**4)*(dy**2)/12.
          rtr(8,8)=rtr(8,8) + (dy**6)/36.
          rtr(8,9)=rtr(8,9) + (dx**2)*(dy**4)/12.
          rtr(8,10)=rtr(8,10) + dx*(dy**5)/12.
          rta(7)=rta(7) + (dx**3)*at(ict)/6.
          rta(8)=rta(8) + (dy**3)*at(ict)/6.
          rta(9)=rta(9) + (dx**2)*dy*at(ict)/2.
          rta(10)=rta(10) + (dy**2)*dx*at(ict)/2.
 20     continue
        rtr(1,1)=float(np)
        rtr(2,1)=rtr(1,2)
        rtr(3,2)=rtr(2,3)
        rtr(3,1)=rtr(1,3)
        if(nds.eq.1) go to 21
        rtr(1,4)=rtr(2,2)/2.
        rtr(1,5)=rtr(3,3)/2.
        rtr(1,6)=rtr(2,3)
        rtr(2,6)=rtr(3,4)*2.
        rtr(3,6)=rtr(2,5)*2.
        rtr(4,5)=rtr(6,6)/4.
        rtr(4,1)=rtr(1,4)
        rtr(4,2)=rtr(2,4)
        rtr(4,3)=rtr(3,4)
        rtr(5,1)=rtr(1,5)
        rtr(5,2)=rtr(2,5)
        rtr(5,3)=rtr(3,5)
        rtr(5,4)=rtr(4,5)
        rtr(6,1)=rtr(1,6)
        rtr(6,2)=rtr(2,6)
        rtr(6,3)=rtr(3,6)
        rtr(6,4)=rtr(4,6)
        rtr(6,5)=rtr(5,6)
        if(nds.ne.3) go to 21
        rtr(1,7)=rtr(2,4)/3.
        rtr(1,8)=rtr(3,5)/3.
        rtr(1,9)=rtr(2,6)/2.
        rtr(1,10)=rtr(2,5)
        rtr(2,7)=2.*rtr(4,4)/3.
        rtr(2,8)=rtr(5,6)/3.
        rtr(2,9)=rtr(4,6)
        rtr(2,10)=2.*rtr(4,5)
        rtr(3,7)=rtr(4,6)/3.
        rtr(3,8)=2.*rtr(5,5)/3.
        rtr(3,9)=2.*rtr(4,5)
        rtr(3,10)=3.*rtr(2,8)
        rtr(5,7)=rtr(4,10)/3.
        rtr(5,9)=rtr(4,8)/3.
        rtr(6,8)=2.*rtr(5,10)/3.
        rtr(6,9)=2.*rtr(4,10)
        rtr(6,10)=2.*rtr(5,9)
        rtr(7,1)=rtr(1,7)
        rtr(7,2)=rtr(2,7)
        rtr(7,3)=rtr(3,7)
        rtr(7,4)=rtr(4,7)
        rtr(7,5)=rtr(5,7)
        rtr(7,6)=rtr(6,7)
        rtr(8,1)=rtr(1,8)
        rtr(8,2)=rtr(2,8)
        rtr(8,3)=rtr(3,8)
        rtr(8,4)=rtr(4,8)
        rtr(8,5)=rtr(5,8)
        rtr(8,6)=rtr(6,8)
        rtr(8,7)=rtr(7,8)
        rtr(9,1)=rtr(1,9)
        rtr(9,2)=rtr(2,9)
        rtr(9,3)=rtr(3,9)
        rtr(9,4)=rtr(4,9)
        rtr(9,5)=rtr(5,9)
        rtr(9,6)=rtr(6,9)
        rtr(9,7)=rtr(7,9)
        rtr(9,8)=rtr(8,9)
        rtr(9,9)=3.*rtr(7,10)
        rtr(9,10)=9.*rtr(7,8)
        rtr(10,1)=rtr(1,10)
        rtr(10,2)=rtr(2,10)
        rtr(10,3)=rtr(3,10)
        rtr(10,4)=rtr(4,10)
        rtr(10,5)=rtr(5,10)
        rtr(10,6)=rtr(6,10)
        rtr(10,7)=rtr(7,10)
        rtr(10,8)=rtr(8,10)
        rtr(10,9)=rtr(9,10)
        rtr(10,10)=3.*rtr(8,9)
 21     continue
c 
c --- invert rtr and store in rtri, na is the row dimension of rtr
c --- and rtri, det is the determinant, nd is the number of coeff.
c --- to be estimated, i.e. the number of independent variables in
c --- rtr, iarea is a work area and ier is an error code. if ier
c --- is greater than 0, there is an inversion error. in this case,
c --- perform a nearest-neighbor interpolation
c
        call f_invmtx(rtr,na,rtri,na,nd,det,iarea,ier)
        if(ier.gt.0) go to 145
c
c --- calculate the regression coeff. - parm
c
        do i=1,nd
          do j=1,nd
            parm(i)=parm(i) + rtri(i,j)*rta(j)
          enddo
        enddo
c
        parm(11)=xo
        parm(12)=yo
 85     continue
c
c --- interpolate the field to the float location
        dxflt=fllon2-parm(11)
        dyflt=fllat2-parm(12)
        vrbint=parm(1)+parm(2)*dxflt+parm(3)*dyflt
     &        +parm(4)*(dxflt**2)/2.+parm(5)*(dyflt**2)/2.+
     &         parm(6)*dxflt*dyflt
        return
c
      endif ! 2-d interpolation
c
 145  continue
      if (intpfl.ne.0 .or. ier.gt.0) then
c
c --- nearest neighbor interpolation if the number of available water
c --- points is smaller than nptmin or if the polynomial interpolation failed.
c --- variable xt is inverse distance
c
        wghts=0.
        do ict=1,np
          xt(ict)=1.0/sqrt(max(1.0e-20,(xt(ict)-fllon2)**2+
     &                                 (yt(ict)-fllat2)**2))
          wghts=wghts+xt(ict)
cdiag     write(6,102) ngrid,ngood,i,j,ict,at(ict),xt(ict)
 102      format('nearest neighbor - ngrid,ngood,i,j,ict,at,xt:'
     &           /10x,5i4,1p,2e13.5)
        enddo
c
        suma=0.
        do ict=1,np
          suma=suma+at(ict)*xt(ict)
        enddo
        vrbint=suma/wghts
        return
c
      endif !nearest neighbor interpolation
c
      end subroutine intrph

      end module mod_floats
c
c
      SUBROUTINE F_INVMTX (A,NA,V,NV,N,D,IP,IER)
c
c --- this matrix inversion code is used in the parameter matrix
c --- objective analysis algorithm of mariano and brown (1994), specifically
c --- in the code used to calculate the large scale 2-d trend surface
c
      INTEGER NA,NV,N,IP(2*N),IER
      DIMENSION A(NA,N),V(NV,N)
      DATA IEXMAX/75/
  115 FORMAT(28H0*MATRIX SINGULAR IN INVMTX*)
  116 FORMAT(34H0*DETERMINANT TOO LARGE IN INVMTX*)
      IER = IERINV_F(N,NA,NV)
      IF (IER .NE. 0) RETURN
      DO 102 J=1,N
         IP(J) = 0
         DO 101 I=1,N
            V(I,J) = A(I,J)
  101    CONTINUE
  102 CONTINUE
      D = 1.
      IEX = 0
      DO 110 M=1,N
         VMAX = 0.
         DO 104 J=1,N
            IF (IP(J) .NE. 0) GO TO 104
            DO 103 I=1,N
               IF (IP(I) .NE. 0) GO TO 103
               VH =ABS(V(I,J))
               IF (VMAX .GT. VH) GO TO 103
               VMAX = VH
               K = I
               L = J
  103       CONTINUE
  104    CONTINUE
         IP(L) = K
         NPM = N+M
         IP(NPM) = L
         D = D*V(K,L)
  105    IF (ABS(D) .LE. 1.0) GO TO 106
         D = D*0.1
         IEX = IEX+1
         GO TO 105
  106    CONTINUE
         PVT = V(K,L)
         IF (M .EQ. 1) PVTMX = ABS(PVT)
         IF (ABS(PVT/FLOAT(M))+PVTMX .EQ. PVTMX) GO TO 113
         V(K,L) = 1.
         DO 107 J=1,N
            HOLD = V(K,J)
            V(K,J) = V(L,J)
            V(L,J) = HOLD/PVT
  107    CONTINUE
         DO 109 I=1,N
            IF (I .EQ. L) GO TO 109
            HOLD = V(I,L)
            V(I,L) = 0.
            DO 108 J=1,N
               V(I,J) = V(I,J)-V(L,J)*HOLD
  108       CONTINUE
  109    CONTINUE
  110 CONTINUE
      M = N+N+1
      DO 112 J=1,N
         M = M-1
         L = IP(M)
         K = IP(L)
         IF (K .EQ. L) GO TO 112
         D = -D
         DO 111 I=1,N
            HOLD = V(I,L)
            V(I,L) = V(I,K)
            V(I,K) = HOLD
  111    CONTINUE
  112 CONTINUE
      IF (IEX .GT. IEXMAX) GO TO 114
      D = D*10.**IEX
      RETURN
  113 IER = 33
c      PRINT 115
      RETURN
  114 IER = 1
      D = FLOAT(IEX)
      PRINT 116
      RETURN
      END SUBROUTINE F_INVMTX
c
      INTEGER FUNCTION IERINV_F (N,NA,NV)
  103 FORMAT(23H0* N .LT. 1 IN INVMTX *)
  104 FORMAT(24H0* NA .LT. N IN INVMTX *)
  105 FORMAT(24H0* NV .LT. N IN INVMTX *)
      IERINV_F = 0
      IF (N .GE. 1) GO TO 101
      IERINV_F = 34
      PRINT 103
      RETURN
  101 IF (NA .GE. N) GO TO 102
      IERINV_F = 35
      PRINT 104
      RETURN
  102 IF (NV .GE. N) RETURN
      IERINV_F = 36
      PRINT 105
      RETURN
      END FUNCTION IERINV_F

      subroutine f_stat(ser,ls,amean,var,std)
c
c --- computes mean, variance, standard deviation of data sequence
      dimension ser(*)
      sum=0.0
      do j=1,ls
        sum=sum+ser(j)
      enddo
      amean=sum/ls
      sum=0.0
      do j=1,ls
        value=ser(j)-amean
        sum=sum+value*value
      enddo
      var=sum/ls
      std=sqrt(var)
c
      return
      end subroutine f_stat
c
c
c --- following are subprograms provided by Geoff Samuels, and inserted
c --- by Nasseer Idrisi (10/5/01)

      subroutine rantab_ini(rtab,iseed1,iseed2,iflag)
c
      real*4     rtab(200,2)
      integer*2 iseed1,iseed2
      integer*4 iflag
c
c --- get the two seeds
c --- iseed1 is for the 'usual'    random number generator  (udev_rt)
c --- iseed2 is for the 'gaussian' random number generator (grand_rt)
c
      integer*4 seed
      integer*2 isd2(2)
      equivalence (isd2,seed)
c
      call system_clock(inisee)
cdiag write(*,*)'calc itime',time()
      itime = inisee
      seed = 123456789
      iseed1 = isd2(1)
      if (iseed1 .lt. 0) then
        iseed1 = iabs (iseed1)
      endif
      iseed1 = iseed1 + itime*iflag
cdiag write(*,*)'iseed1',iseed1
      seed = 987654321
      iseed2 = isd2(2)
      if (iseed2 .lt. 0) then
        iseed2 = iabs (iseed1)
      endif
      iseed2 = iseed2 + itime*iflag
cdiag write(*,*)'iseed2',iseed2
c
c --- set seeds to negative values if iflag=1
c
      if (iflag .eq. 1) then
        iseed1 = -iseed1
        iseed2 = -iseed2
      endif
c
c --- load the table with a sequence of 200 random numbers.
      do  ic = 1,200
        rtab(ic,1) =  udev_rt(iseed1)
        rtab(ic,2) = grand_rt(iseed2)
      enddo
cdiag write(*,*)'rtabs',rtab(100,1),rtab(100,2)
c
      return
      end subroutine rantab_ini

      subroutine rantab (rtab,iseed1,iseed2,numran)
c
c --- each time this routine is called one of the 200 records in "rtab"
c --- is randomly selected and new random numbers are inserted
c
      real*4 rtab(200,2)
c
c --- pick a number between 1 and 200
      numran = nint(1+199*udev_rt(iseed1))
      if (numran .gt. 200) then
        numran = 200
      endif
c
c --- get new random numbers
      rtab(numran,1) =  udev_rt(iseed1)
      rtab(numran,2) = grand_rt(iseed2)
c
      return
      end subroutine rantab
c
      real function grand_rt(idum)
c
c provides random number from gaussian distribution, mean 0, varaince 1,
c for finding random component of turbulent u
c
c from arthur mariano using routines from numerical recipes
c
      data iset/0/
      save iset,gset
c
      if (iset.eq.0) then
 1      v1=2.*udev_rt(idum)-1.0
        v2=2.*udev_rt(idum)-1.0
        r=v1**2+v2**2
        if (r.ge.1.0)go to 1
        fac=sqrt(-2.*log(r)/r)
        gset=v1*fac
        grand_rt=v2*fac
        iset=1
      else
        grand_rt=gset
        iset=0
      endif
c
      return
      end function grand_rt
c
      real function udev_rt(iseed)

      parameter (m=714025,ia=1366,ic=150889,rm=1./m)

      dimension ir(97)

      data iff / 0 /
c
      if (iseed .lt. 0 .or. iff .eq. 0) then
        iff = 1
        iseed = mod(ic-iseed,m)
        do j=1,97
          iseed = mod(ia*iseed+ic,m)
          ir(j) = iseed
        enddo
        iseed = mod(ia*iseed+ic,m)
      else
        iseed = mod(iseed,m)
      endif
      iy=iseed
      j = min(97,1+(97*iy)/m)
      iy = ir(j)
      udev_rt = iy*rm
      iseed = mod(ia*iseed+ic,m)
      ir(j) = iseed
      return
      end function udev_rt
c
c
c> Revision history:
c>
c> Nov  2006 - 1st module version
