      program altim_usgodae
c
c --- test altimetry representation error models
c
      real*4 rtab(200,2)
      integer*4 seed,numran,inisee,iflag
      integer*2 iseed1,iseed2
      equivalence ( iseed1, iseed2, seed )
c
      data      nsave/10/
c
      call xcspmd
      call zaiost
      lp=6
c
c --- initialize sshas and dtimes for spatially correlated errors
c
      do l=1,nsave
        ptnums(l)=0
        alons(l)=0.0
        alats(l)=0.0
        sshas(l)=0.0
        dtimes(l)=0.0
      enddo
c
c --- initialize random seeds and create random number table
c --- if iflag=1 (iflag=0), then time() is (is not) used to generate seeds
c
        iflag=1 
        call system_clock(inisee)
        seed = 414957000-inisee
        iede = iseed1
        ied = iseed2
        if (iede .lt. 0) iede = iabs(iede)
        if (ied .lt. 0) ied = iabs(ied)
        call rantab_ini(rtab,iseed1,iseed2,iflag)
c
c --- read model data
c ---   'flnm  ' = name of file containing the actual data
c ---   'iexpt ' = experiment number x10  (000=from archive file)
c ---   'yrflag' = days in year flag (0=360J16,1=366J16,2=366J01,3=actual)
c ---   'iyear ' = year of model run
c ---   'jday  ' = julian day of the model run year
c ---   'idm   ' = longitudinal array size
c ---   'jdm   ' = latitudinal  array size
c ---   'kdm   ' = number of layers
c
        read (*,'(a)') flnm
        write (lp,'(2a)') ' input archive file: ',trim(flnm)
        call flush(lp)
c
        call blkini(iexpt, 'iexpt ')
        call blkini(yrflag,'yrflag')
        call blkini(iyear, 'iyear ')
        call blkini(jday,  'jday  ')
        if     (yrflag.eq.3) then
          call jday1900(iyear,jday,jd)
        elseif (yrflag.eq.1 .or. yrflag.eq.2) then
          jd = (iyear-1)*366+jday
        elseif (yrflag.eq.0) then
          jd = (iyear-1)*360+jday
        endif
c
        call blkini(ii,    'idm   ')
        call blkini(jj,    'jdm   ')
        call blkini(kk,    'kdm   ')
        if     (ii.ne.idm .or. jj.ne.jdm) then
          write(lp,*)
          write(lp,*) 'error - wrong idm or jdm (should be:',
     &                                           idm,jdm,')'
          write(lp,*)
          call flush(lp)
          stop
        endif
c
c ---   'thbase' = reference density (sigma units)
        call blkinr(thbase,
     &             'thbase','("blkinr: ",a6," =",f11.4," sig")')
c
c ---   'mthin ' = mask thin layers from output
        call blkinl(mthin, 'mthin ')
c
c ---   'iorign' = i-origin of sampled subregion
c ---   'jorign' = j-origin of sampled subregion
c ---   'idmp  ' = i-extent of sampled subregion (<=idm; 0 implies idm)
c ---   'jdmp  ' = j-extent of sampled subregion (<=jdm; 0 implies jdm)
        call blkini(iorign,'iorign')
        call blkini(jorign,'jorign')
        call blkini(ii,    'idmp  ')
        call blkini(jj,    'jdmp  ')
        if     (ii.eq.0) then
          ii=idm
        endif
        if     (jj.eq.0) then
          jj=jdm
        endif
c
c ---   'iorign,jorign' denote the origin of the subgrid to be extracted 
c ---   from the full history grid (dimensioned idm x jdm). 
c ---   The size of the subgrid is determined by ii,jj.
        write (lp,'(2(a,i5),9x,2(a,i5))') 'extracting i =',iorign,
     &    ' ...',iorign+ii-1,'j =',jorign,' ...',jorign+jj-1
        call flush(lp)
c
c ---   'adjlon' = adjust longitude range to be within -180 and +180 degrees
c ---   'lonmin' = minimum longitude
c ---   'lonmax' = maximum longitude
c ---   'latmin' = minimum latitude
c ---   'latmax' = maximum latitude
c ---   'depcut' = altimetry not assimilated shallower than this cutoff depth
c ---   'rnoise' = rms altimeter noise level (m)
c ---   'terror' = internal tide rms error magnitude (m)
c ---   'tscale' = spatial scale of internal tide errors (km)
c ---   'eerror' = unresolved synoptic eddy rms error magnitude (m)
c ---   'escale' = spatial scale of eddy errors (km)
c ---   'altspd' = satellite speed (j1,j2-5.9167, en-6.634)
c
        call blkinr(adjlon,
     &             'adjlon','("blkinr: ",a6," =",f11.4," sig")')
        call blkinr(alomin,
     &             'lonmin','("blkinr: ",a6," =",f11.4," sig")')
        call blkinr(alomax,
     &             'lonmax','("blkinr: ",a6," =",f11.4," sig")')
        call blkinr(alamin,
     &             'latmin','("blkinr: ",a6," =",f11.4," sig")')
        call blkinr(alamax,
     &             'latmax','("blkinr: ",a6," =",f11.4," sig")')
        call blkinr(depcut,
     &             'depcut','("blkinr: ",a6," =",f11.4," sig")')
        call blkinr(rnoise,
     &             'rnoise','("blkinr: ",a6," =",f11.4," sig")')
        call blkinr(terror,
     &             'terror','("blkinr: ",a6," =",f11.4," sig")')
        call blkinr(tscale,
     &             'tscale','("blkinr: ",a6," =",f11.4," sig")')
        call blkinr(eerror,
     &             'eerror','("blkinr: ",a6," =",f11.4," sig")')
        call blkinr(escale,
     &             'escale','("blkinr: ",a6," =",f11.4," sig")')
        call blkinr(altspd,
     &             'altspd','("blkinr: ",a6," =",f11.4," sig")')
        dttscl=(tscale/altspd)/86400.0
        dtescl=(escale/altspd)/86400.0
c
c --- array allocation
c
      call plot_alloc
c
      dpthfil = 'regional.depth'
c
      do j=1,jj
        do i=1,ii
          p(i,j,1)=0.
        enddo
      enddo
c
c --- read the archive file.
c
      if (lhycom) then
        call getdat(flnm,time3,artype,initl,icegln,trcout,
     &              iexpt,iversn,yrflag,kkin)       ! hycom input
        time = time3(3)
      else
        call getdtm(flnm,time,initl, thbase)        ! micom input
        artype = 1
        iversn = 10
      endif
c
      write(lp,'(/a,2f8.2/a,2f8.2)') 
     &     'sub-domain longitude range = ',
     &    minval(plon(:,:)),maxval(plon(:,:)),
     &     'sub-domain latitude  range = ',
     &    minval(plat(:,:)),maxval(plat(:,:))
c
      lperiod = ii.eq.idm .and.
     &          maxval(plon(:,:))-minval(plon(:,:)) .gt. 350.0
      if     (lperiod) then
        write(lp,'(/a/)') 'sub-domain assumed to be periodic'
      else
        write(lp,'(/a/)') 'sub-domain assumed to be non-periodic'
      endif
      call flush(lp)
c
      call bigrid(depths)
      call flush(lp)
c
c --- check that bathymetry is consistent with this archive.
c --- only possible with hycom .[ab] file input.
c
      if     (iversn.ge.20) then
        ibadl = 0
        ibads = 0
        do j= 1,jj
          do i= 1,ii
            if     (ip(i,j).eq.1) then
              if     (srfht(i,j).gt.2.0**99) then
                ibads = ibads + 1   ! topo sea, srfht land
              endif
            else
              if     (srfht(i,j).lt.2.0**99) then
                ibadl = ibadl + 1   ! topo land, srfht sea
              endif
            endif
          enddo
        enddo
        if     (ibads.ne.0) then
          write(lp,*)
          write(lp,*) 'error - wrong bathymetry for this archive file'
          write(lp,*) 'number of topo sea  mismatches = ',ibads
          write(lp,*) 'number of topo land mismatches = ',ibadl
          write(lp,*)
          call flush(lp)
          stop
        endif
        if     (ibadl.ne.0 .and. lhycom) then
          write(lp,*)
          write(lp,*) 'error - wrong bathymetry for this archive file'
          write(lp,*) 'number of topo sea  mismatches = ',ibads
          write(lp,*) 'number of topo land mismatches = ',ibadl
          write(lp,*)
          call flush(lp)
          stop
        endif
      endif !iversn.ge.20
c
      call bigrd1(depths)
c
      do j=1,jj
        do i=1,ii
          if     (ip(i,j).eq.1) then
            srfht(i,j )= srfht(i,j)/9.806  ! m
            p(i,j,kk+1) = depths(i,j)
          else
            srfht(i,j) = flag
            p(i,j,kk+1) = flag
          endif
        enddo
      enddo
c
c --- remove nature run mdt from srfht
c
      read(20,*) narchv
      do j=1,jj
        do i=1,ii
          read(20,*) sshavg
          if (srfht(i,j).ne.flag) then
            srfht(i,j) = srfht(i,j)-sshavg
          endif
        enddo
      enddo
c
c --- -----------------------------------------------------
c --- extract ssh anomaly data along altimetry track and add errors
c --- -----------------------------------------------------
c
c --- sample synthetic ssh
c
      read(21,*) nptsin
      write(22,*) nptsin
c
      icount=0
      do i=1,nptsin
        read(21,*) ptnum,alat,alon,dtime,value
        alon2 = alon
        alat2 = alat
        alon2 = alon2+adjlon
c
        if(alon.ge.alomin .and. alon.le.alomax .and.
     &     alat.ge.alamin .and. alat.le.alamax) then
c
          call sample(alon2,alat2,depcut,flag,srfht,iloc,jloc,ssha)
c
          if (ssha.ne.flag) then
c
            icount=icount+1
c
c ---       local random errors
c
c ---       add random gaussian measurement errors as local noise
            if (rnoise.gt.0.0) then
              call rantab(rtab,iseed1,iseed2,numran)
              ssha=ssha+rnoise*rtab(numran,2)
            endif
c
c ---       representation errors
c
c ---       first save previous ssha estimates
c
            do l=nsave-1,1,-1
              ptnums(l+1)=ptnums(l)
              alons(l+1)=alons(l)
              alats(l+1)=alats(l)
              sshas(l+1)=sshas(l)
              dtimes(l+1)=dtimes(l)
            enddo
            ptnums(1)=ptnum
            alons(1)=alon
            alats(1)=alat
            sshas(1)=ssha
            dtimes(1)=dtime
c
c ---       add internal tide errors to the synthetic SSHA
c ---       this is a representation error with space-time correlation scales,
c ---       so this simple model of the error is not realistic and needs to
c ---       be upgraded
            if (terror.gt.0.0) then
              itlag=1
              do l=2,nsave
                if(dtime-dtimes(l).lt.dttscl. and.
     &             dtime-dtimes(l).gt.0.0) then
                  itlag=itlag+1
                endif
              enddo
              call rantab(rtab,iseed1,iseed2,numran)
              errmag=terror*rtab(numran,2)/itlag
              do l=1,itlag
                sshas(l)=sshas(l)+errmag
              enddo
            endif
c
c ---       add unresolved synoptic eddy errors to the synthetic SSHA
c ---       this is a representation error with space-time correlation scales,
c ---       so this simple model of the error is not realistic and needs to
c ---       be upgraded
            if (eerror.gt.0.0) then
              itlag=1
              do l=2,nsave
                if(dtime-dtimes(l).lt.dtescl. and.
     &             dtime-dtimes(l).gt.0.0) then
                  itlag=itlag+1
                endif
              enddo
              call rantab(rtab,iseed1,iseed2,numran)
              errmag=eerror*rtab(numran,2)/itlag
              do l=1,itlag 
                sshas(l)=sshas(l)+errmag
              enddo
            endif
c
c --- store observations
c
            if (icount.ge.nsave) then
              isave=icount-nsave+1
              ptnumo(isave) = ptnums(nsave)
              alono(isave)  = alons(nsave)
              alato(isave)  = alats(nsave)
              dtimeo(isave) = dtimes(nsave)
              sshao(isave)  = sshas(nsave)
            endif
c
          endif
c
        endif
c
      enddo ! nptsin
c
c --- save the final nsave data samples
c
      do l=min(icount,nsave-1),1,-1
        isave=icount-l+1
        ptnumo(isave) = ptnums(l)
        alono(isave)  = alons(l)
        alato(isave)  = alats(l)
        dtimeo(isave) = dtimes(l)
        sshao(isave)  = sshas(l)
      enddo
c
c --- write observations to assimilation input file
c
      do i=1,nptsin
        write(22,910) ptnumo(i),alato(i),alono(i),dtimeo(i),sshao(i)
  910   format(f8.1,f11.6,f12.6,f13.6,f10.6)
      enddo
c
      end
