      program xctd
      use mod_plot  ! HYCOM plot array interface
      use mod_za    ! HYCOM array I/O interface
c
c --- hycom/micom xctd profile extractor
c --- used to sample nature runs and add errors for osses
c
c --- number of z layers
c
      integer, parameter :: ikdm = 64, izdm = 271, isdm = 50000
c
      character flnm*240,frmt*80,cline*240,sigfmt*40
      logical   mthin,icegln,lperiod,baclin,barouv
c
      integer artype,iexpt,iversn,kkin,yrflag,jday,jd,iloc,jloc,thkflg
      double precision time3(3)
c
      real dp0k(ikdm),ds0k(ikdm),dssk(ikdm),dp0ij(ikdm),dp0cum(ikdm),
     &     pcen(0:ikdm),sigma(ikdm)
      real dpm,dpms,dsm,dsms,q
      real alono(isdm),alato(isdm),zdepo(isdm),tempo(isdm),
     &     salno(isdm),time(isdm)
c
      real, parameter :: flag = -9999.0
c
c
c --- convert depth rms error into T and S errors
c
      real*4    rtab(200,2)
      real*4,    allocatable, dimension (:,:,:) ::
     &   tempz,salnz
      real*4,    allocatable, dimension (:,:)   ::
     &   templv,salnlv
      real*4    zmax,zdep(izdm)
      integer*4 seed,numran,inisee,iflag,thflag
      integer*2 iseed1,iseed2
      equivalence ( iseed1, iseed2, seed )
c
c --- 'lhycom' -- hycom (vs micom) input file
c --- 'trcout' -- tracer input
      logical   lhycom,trcout
      data      lhycom/.true. /, trcout/.false./
c
      real      tenm,onem,temcm,onecm,onemm
      data      tenm/10./,onem/1./,tencm/.1/,onecm/.01/,onemm/.001/
c
      logical   initl
      data      initl /.true. /
c
      if     (thflag.eq.0) then
        sigfmt = '(a6," =",f10.4," sigma-0")'
      elseif (thflag.eq.2) then
        sigfmt = '(a6," =",f10.4," sigma-2")'
      endif
c
      call xcspmd
      call zaiost
      lp=6
c
c --- initialize random seeds and create random number table
c --- if iflag=1 (iflag=0), then time() is (is not) used to generate seeds
c
        iflag=1 
        call system_clock(inisee)
        seed = 414957000-inisee
        iede = iseed1
        ied = iseed2
        if (iede .lt. 0) iede = iabs(iede)
        if (ied .lt. 0) ied = iabs(ied)
        call rantab_ini(rtab,iseed1,iseed2,iflag)
c
c --- read model data
c ---   'flnm  ' = name of file containing the actual data
c ---   'iexpt ' = experiment number x10  (000=from archive file)
c ---   'yrflag' = days in year flag (0=360J16,1=366J16,2=366J01,3=actual)
c ---   'iyear ' = year of model run
c ---   'jday  ' = julian day of the model run year
c ---   'idm   ' = longitudinal array size
c ---   'jdm   ' = latitudinal  array size
c ---   'kdm   ' = number of layers
        read (*,'(a)') flnm
        write (lp,'(2a)') ' input file: ',trim(flnm)
        call flush(lp)
c
        call blkini(iexpt, 'iexpt ')
        call blkini(yrflag,'yrflag')
        call blkini(iyear, 'iyear ')
        call blkini(jday,  'jday  ')
        if     (yrflag.eq.3) then
          call jday1900(iyear,jday,jd)
        elseif (yrflag.eq.1 .or. yrflag.eq.2) then
          jd = (iyear-1)*366+jday
        elseif (yrflag.eq.0) then
          jd = (iyear-1)*360+jday
        endif
c
        call blkini(ii,    'idm   ')
        call blkini(jj,    'jdm   ')
        call blkini(kk,    'kdm   ')
        if     (ii.ne.idm .or. jj.ne.jdm) then
          write(lp,*)
          write(lp,*) 'error - wrong idm or jdm (should be:',
     &                                           idm,jdm,')'
          write(lp,*)
          call flush(lp)
          stop
        endif
C
C ---   'nhybrd' = number of hybrid levels (0=all isopycnal)
C ---   'nsigma' = number of sigma  levels (nhybrd-nsigma z-levels)
C ---   'thkflg' = 0 to calculate, 1 to read in, dp0k(k) and ds0k(k)
C
        call blkini(nhybrd,'nhybrd')
        call blkini(nsigma,'nsigma')
        call blkini(thkflg,'thkflg')
C
        if (thkflg.eq.0) then   ! calculate dp0k(k),ds0k(k)
C
C ---     'dp00'   = deep    z-level spacing minimum thickness (m)
C ---     'dp00x'  = deep    z-level spacing maximum thickness (m)
C ---     'dp00f'  = deep    z-level spacing stretching factor (1.0=const.z)
C ---     'ds00'   = shallow z-level spacing minimum thickness (m)
C ---     'ds00x'  = shallow z-level spacing maximum thickness (m)
C ---     'ds00f'  = shallow z-level spacing stretching factor (1.0=const.z)
C
C ---     the above describe a system that is isopycnal or:
C ---        z in    deep water, based on dp00,dp00x,dp00f
C ---        z in shallow water, based on ds00,ds00x,ds00f and nsigma
C ---        sigma between them, based on ds00,ds00x,ds00f and nsigma
C
C ---     for z-only set nsigma=0 (and ds00,ds00x,ds00f=dp00,dp00x,dp00f)
C ---     for sigma-z (shallow-deep) use a very small ds00
C ---      (pure sigma-z also has ds00f=dp00f and ds00x=dp00x*ds00/dp00)
C ---     for z-sigma (shallow-deep) use a very large dp00 (not recommended)
C ---     for sigma-only set nsigma=kdm, dp00 large, and ds00 small
C
          CALL BLKINR(DP00,  'dp00  ','(a6," =",f10.4," m")')
          CALL BLKINR(DP00X, 'dp00x ','(a6," =",f10.4," m")')
          CALL BLKINR(DP00F, 'dp00f ','(a6," =",f10.4," ")')
          CALL BLKINR(DS00,  'ds00  ','(a6," =",f10.4," m")')
          CALL BLKINR(DS00X, 'ds00x ','(a6," =",f10.4," m")')
          CALL BLKINR(DS00F, 'ds00f ','(a6," =",f10.4," ")')
          call blkinr(dp00i, 'dp00i ','(a6," =",f10.4," m")')
C
          IF (NSIGMA.LE.1) THEN
            NSIGMA=1
            DS00  =DP00
            DS00X =DP00X
            DS00F =DP00F
          ENDIF
C
        ELSEIF (thkflg.eq.1) then   ! read in dp0k(k),ds0k(k)
C
          do k=1,nhybrd
            call blkinr(dp0k(k),
     &                  'dp0k  ','("blkinr: ",a6," =",f11.4," m")')
            dp0k(k)=dp0k(k)*onem
          enddo
          do k=1,nsigma
            call blkinr(ds0k(k),
     &                  'ds0k  ','("blkinr: ",a6," =",f11.4," m")')
            ds0k(k)=ds0k(k)*onem
          enddo
C
        ENDIF    ! calculate or read in dp0k(k),ds0k(k)
c
c ---   'thflag' = reference pressure flag (0=Sigma-0, 2=Sigma-2, 4=Sigma-4)
c ---   'thbase' = reference density (sigma units)
        call blkini(thflag,'thflag')
        call blkinr(thbase,
     &             'thbase','("blkinr: ",a6," =",f11.4," sig")')
c
        if     (thflag.eq.0) then
          sigfmt = '(a6," =",f10.4," sigma-0")'
        elseif (thflag.eq.2) then
          sigfmt = '(a6," =",f10.4," sigma-2")'
        endif
c
C --- 'sigma ' = isopycnal layer target densities (sigma units)
        DO k=1,kk
          CALL BLKINR(SIGMA(K),'sigma ',sigfmt)
          IF     (K.GT.1) THEN
            IF     (SIGMA(K).LT.SIGMA(K-1)) THEN
              WRITE(6,*)
              WRITE(6,*) 'ERROR - sigma(k) must be > sigma(k-1)'
              WRITE(6,*)
              CALL ZHFLSH(6)
              STOP
            ENDIF
          ENDIF
        ENDDO
c
c ---   'mthin ' = mask thin layers from output
        call blkinl(mthin, 'mthin ')
c
c ---   'iorign' = i-origin of sampled subregion
c ---   'jorign' = j-origin of sampled subregion
c ---   'idmp  ' = i-extent of sampled subregion (<=idm; 0 implies idm)
c ---   'jdmp  ' = j-extent of sampled subregion (<=jdm; 0 implies jdm)
        call blkini(iorign,'iorign')
        call blkini(jorign,'jorign')
        call blkini(ii,    'idmp  ')
        call blkini(jj,    'jdmp  ')
        if     (ii.eq.0) then
          ii=idm
        endif
        if     (jj.eq.0) then
          jj=jdm
        endif
c
c ---   'iorign,jorign' denote the origin of the subgrid to be extracted 
c ---   from the full history grid (dimensioned idm x jdm). 
c ---   The size of the subgrid is determined by ii,jj.
        write (lp,'(2(a,i5),9x,2(a,i5))') 'extracting i =',iorign,
     &    ' ...',iorign+ii-1,'j =',jorign,' ...',jorign+jj-1
        call flush(lp)
c
c ---   'lonmin' = minimum longitude
c ---   'lonmax' = maximum longitude
c ---   'latmin' = minimum latitude
c ---   'latmax' = maximum latitude
c ---   'depcut' = xctds not assimilated shallower than this cutoff depth
c ---   'trnois' = rms noise level of synthetic temperature
c ---   'srnois' = rms noise level of synthetic salinity
c ---   'tenois' = unresolved temperature synoptic eddy noise level
c ---   'senois' = unresolved salinity synoptic eddy noise level
c ---   'deperr' = rms depth error (percent of depth)
c
        call blkinr(alomin,
     &             'lonmin','("blkinr: ",a6," =",f11.4," sig")')
        call blkinr(alomax,
     &             'lonmax','("blkinr: ",a6," =",f11.4," sig")')
        call blkinr(alamin,
     &             'latmin','("blkinr: ",a6," =",f11.4," sig")')
        call blkinr(alamax,
     &             'latmax','("blkinr: ",a6," =",f11.4," sig")')
        call blkinr(depcut,
     &             'depcut','("blkinr: ",a6," =",f11.4," sig")')
        call blkinr(trnois,
     &             'trnois','("blkinr: ",a6," =",f11.4," sig")')
        call blkinr(srnois,
     &             'srnois','("blkinr: ",a6," =",f11.4," sig")')
        call blkinr(tenois,
     &             'tenois','("blkinr: ",a6," =",f11.4," sig")')
        call blkinr(senois,
     &             'senois','("blkinr: ",a6," =",f11.4," sig")')
        call blkinr(deperr,
     &             'deperr','("blkinr: ",a6," =",f11.4," sig")')
c
c --- array allocation
c
      call plot_alloc
c
      dpthfil = 'regional.depth'
c
c --- calculate dp0k and ds0k
c
      if (thkflg.eq.0) then
c ---   logorithmic k-dependence of dp0 (deep z's)
        dp0k(1) = dp00
        dpm     = dp0k(1)
        dpms    = dpm
c
        dp0kf=1.0
        do k=2,kk
          dp0kf=dp0kf*dp00f
          if     (k.le.nhybrd) then
            if     (dp00f.ge.1.0) then
              dp0k(k)=min(dp00*dp0kf,dp00x)
            else
              dp0k(k)=max(dp00*dp0kf,dp00x)
            endif
          else
            dp0k(k)=0.0
          endif
          dpm  = dp0k(k)
          dpms = dpms + dpm
        enddo !k
c
c ---   logorithmic k-dependence of ds0 (shallow z-s)
        ds0k(1) = ds00
        dsm     = ds0k(1)
        dsms    = dsm
c
        ds0kf=1.0
        do k=2,nsigma
          ds0kf=ds0kf*ds00f
          if     (ds00f.ge.1.0) then
            ds0k(k)=min(ds00*ds0kf,ds00x)
          else
            ds0k(k)=max(ds00*ds0kf,ds00x)
          endif
          dsm  = ds0k(k)
          dsms = dsms + dsm
        enddo !k
c
c --- sigma-depth scale factors
        do k=1,nsigma
          dssk(k)=ds0k(k)/dsms  ! fraction of depths in sigma layer k
        enddo
        do k= nsigma+1,kk
          ds0k(k)=dp0k(k)
          dssk(k)=0.0           ! these layers are zero in sigma mode
        enddo
c
      endif   ! calculate dp0k(k),ds0k(k)
c
c --- set z levels (2 m to 200 m, 5 m to 1100 m)
c
      k2m=100/2+1
      k5m=1100/5
      nzlev=k2m+k5m
      if (nzlev.gt.izdm) then
        stop 'increase z-level dimension izdm'
      endif
      do k=1,k2m
        zdep(k)=float(k-1)*2.0
      enddo
      do k=1,k5m
        zdep(k+k2m)=zdep(k+k2m-1)+5.0
      enddo
c
c --- array allocation for level fields
c
      allocate(tempz(ii,jj,nzlev) )
      allocate(salnz(ii,jj,nzlev) )
      allocate(templv(ii,jj) )
      allocate(salnlv(ii,jj) )
c
      do j=1,jj
        do i=1,ii
          p(i,j,1)=0.
        enddo
      enddo
c
c --- read the archive file.
c
      if (lhycom) then
        call getdat(flnm,time3,artype,initl,icegln,trcout,
     &              iexpt,iversn,yrflag,kkin)       ! hycom input
        time = time3(3)
      else
        call getdtm(flnm,time,initl, thbase)        ! micom input
        artype = 1
        iversn = 10
      endif
c
      write(lp,'(/a,2f8.2/a,2f8.2)') 
     &     'sub-domain longitude range = ',
     &    minval(plon(:,:)),maxval(plon(:,:)),
     &     'sub-domain latitude  range = ',
     &    minval(plat(:,:)),maxval(plat(:,:))
c
      lperiod = ii.eq.idm .and.
     &          maxval(plon(:,:))-minval(plon(:,:)) .gt. 350.0
      if     (lperiod) then
        write(lp,'(/a/)') 'sub-domain assumed to be periodic'
      else
        write(lp,'(/a/)') 'sub-domain assumed to be non-periodic'
      endif
      call flush(lp)
c
      call bigrid(depths)
      call flush(lp)
c
c --- check that bathymetry is consistent with this archive.
c --- only possible with hycom .[ab] file input.
c
      if     (iversn.ge.20) then
        ibadl = 0
        ibads = 0
        do j= 1,jj
          do i= 1,ii
            if     (ip(i,j).eq.1) then
              if     (srfht(i,j).gt.2.0**99) then
                ibads = ibads + 1   ! topo sea, srfht land
              endif
            else
              if     (srfht(i,j).lt.2.0**99) then
                ibadl = ibadl + 1   ! topo land, srfht sea
              endif
            endif
          enddo
        enddo
        if     (ibads.ne.0) then
          write(lp,*)
          write(lp,*) 'error - wrong bathymetry for this archive file'
          write(lp,*) 'number of topo sea  mismatches = ',ibads
          write(lp,*) 'number of topo land mismatches = ',ibadl
          write(lp,*)
          call flush(lp)
          stop
        endif
        if     (ibadl.ne.0 .and. lhycom) then
          write(lp,*)
          write(lp,*) 'error - wrong bathymetry for this archive file'
          write(lp,*) 'number of topo sea  mismatches = ',ibads
          write(lp,*) 'number of topo land mismatches = ',ibadl
          write(lp,*)
          call flush(lp)
          stop
        endif
      endif !iversn.ge.20
c
      call bigrd1(depths)
c
      do k=1,kkin
        do j=1,jj
          do i=1,ii
c
c --- convert layer thickness to meters
            if     (ip(i,j).eq.1) then
              dp(i,j,k)=dp(i,j,k)/9806.
              p(i,j,k+1)=p(i,j,k)+dp(i,j,k)
            else
              saln(i,j,k)=flag
              temp(i,j,k)=flag
              p(i,j,k+1)=flag
            endif
          enddo
        enddo
      enddo
c
      do j=1,jj
        do i=1,ii
          if (kkin.eq.1 .or. kkin.lt.kk) then
            if     (ip(i,j).eq.1) then
              p(i,j,kk+1)=depths(i,j)
            else
              p(i,j,kk+1)=flag
            endif
          endif
        enddo
      enddo
c
c --- -----------------------------------
c --- generate the z-level T and S arrays
c --- -----------------------------------
c
      call layer2z(temp,p,tempz,zdep,flag,ii,jj,kk,nzlev,1)
      call layer2z(saln,p,salnz,zdep,flag,ii,jj,kk,nzlev,1)
c
c --- -------------------------------------
c --- extract xctd T profiles and add errors
c --- -------------------------------------
c
c --- sample profiles
c
      ifirst = 0
c
   77 continue
c
      read(51,*,end=78) nptsin
c
      icount=0
      do iii=1,nptsin
        read(51,*) alon,alat,depth,dtime,value
        if (alon.gt.180.0) then
          alon2 = alon-360.0
        elseif (alon.lt.-180.0) then
          alon2 = alon+360.0
        else
          alon2=alon
        endif
        alat2 = alat
c
        if(alon2.ge.alomin .and. alon2.le.alomax .and.
     &     alat2.ge.alamin .and. alat2.le.alamax) then
c
c --- calculate random depth error for each new profile
          derr=0.0
          if (deperr.gt.0.0 .and. iii.eq.1) then
            call rantab(rtab,iseed1,iseed2,numran)
            derr=deperr*rtab(numran,2)
          endif
          depthe=max(1.0,depth+derr*min(1.0,depth/100.0)) ! taper above 100 m
c
c --- find the k and k+1 indices of the z array containing the sample depth
c --- plus error
          q=-1.0
          do k=1,nzlev-1
            if (zdep(k).le.depthe .and. zdep(k+1).gt.depthe) then
              k2=k
              q=(zdep(k+1)-depthe)/(zdep(k+1)-zdep(k))
            endif
          enddo
          if (q.lt.0.0 .or. q.gt.1.0) then
            stop 'bad q - vertical interpolation failed'
          endif
c
c --- calculate the vertically interpolated templv and salnlv  arrays
c --- for input to sample
          do j=1,jj
            do i=1,ii
              templv(i,j)=flag
              salnlv(i,j)=flag
              if (ip(i,j).eq.1 .and. tempz(i,j,k2  ).ne.flag .and.
     &                               tempz(i,j,k2+1).ne.flag .and.
     &                               salnz(i,j,k2  ).ne.flag .and.
     &                               salnz(i,j,k2+1).ne.flag) then
                templv(i,j)=q*tempz(i,j,k2)+(1.0-q)*tempz(i,j,k2+1)
                salnlv(i,j)=q*salnz(i,j,k2)+(1.0-q)*salnz(i,j,k2+1)
              endif
            enddo
          enddo
c
          call sample(alon2,alat2,depcut,flag,templv,iloc,jloc,tsampl)
          call sample(alon2,alat2,depcut,flag,salnlv,iloc,jloc,ssampl)
c
          if (tsampl.ne.flag .and. ssampl.ne.flag) then
c
            icount = icount + 1
            if (icount.gt.isdm) then
              stop 'increase sample dimension isdm'
            endif
c
c --- add random gaussian errors to the synthetic T and S profiles
c --- confine tenois and senois to the upper 100 m
c
            if (trnois.gt.0.0) then
              call rantab(rtab,iseed1,iseed2,numran)
              tsampl = tsampl+trnois*rtab(numran,2)
            endif
            if (srnois.gt.0.0) then
              call rantab(rtab,iseed1,iseed2,numran)
              ssampl = ssampl+srnois*rtab(numran,2)
            endif
            if (tenois.gt.0.0) then
              call rantab(rtab,iseed1,iseed2,numran)
              factor = max(0.0,(100.0-depth)/100.0)
              tsampl = tsampl+tenois*rtab(numran,2)*factor
            endif
            if (senois.gt.0.0) then
              call rantab(rtab,iseed1,iseed2,numran)
              factor = max(0.0,(100.0-depth)/100.0)
              ssampl = ssampl+senois*rtab(numran,2)*factor
            endif
c
c --- store T and S observations
c
            alono(icount) = alon
            alato(icount) = alat
            zdepo(icount) = depth
            time(icount)  = dtime
            tempo(icount) = tsampl
            salno(icount) = ssampl
c
          endif
c
        endif
c
      enddo ! nptsin
c
c
c --- write observations to assimilation input file
c
      ndata = icount
      if (ndata.ge.12) then  ! at least 12 depths sampled
        write(52,900) ndata
  900   format(i8)
        do i=1,ndata
          write(52,910) alono(i),alato(i),zdepo(i),time(i),tempo(i),
     &                  salno(i)
  910     format(f9.3,f8.3,f9.3,f11.4,f7.3,f9.3)
        enddo
      endif
c
      go to 77
c
   78 continue
c
      end
