      program insitusst
      use mod_plot  ! HYCOM plot array interface
      use mod_za    ! HYCOM array I/O interface
c
c --- hycom/micom surface insitu sst temperature data extractor
c
      character flnm*240,frmt*80,cline*240
      logical   mthin,icegln,lperiod,baclin,barouv
c
      integer          artype,iexpt,iversn,kkin,yrflag,iloc,jloc
      double precision time3(3)
c
      real alono(50000),alato(50000),sstao(50000),time(50000)
c
      real, parameter :: flag = 2.0**100
c
      real*4 rtab(200,2)
      integer*4 seed,numran,inisee,iflag
      integer*2 iseed1,iseed2
      equivalence ( iseed1, iseed2, seed )
c
c --- 'lhycom' -- hycom (vs micom) input file
c --- 'trcout' -- tracer input
      logical   lhycom,trcout
      data      lhycom/.true. /, trcout/.false./
c
      real      tenm,onem,temcm,onecm,onemm
      data      tenm/10./,onem/1./,tencm/.1/,onecm/.01/,onemm/.001/
c
      logical   initl
      data      initl /.true. /
      real      thref,spcifh
      data      thref/1.e-3/,spcifh/3990./
      character blank*40
      data      blank/'                                        '/
c
      call xcspmd
      call zaiost
      lp=6
c
c --- initialize random seeds and create random number table
c --- if iflag=1 (iflag=0), then time() is (is not) used to generate seeds
c
        iflag=1 
        call system_clock(inisee)
        seed = 414957000-inisee
        iede = iseed1
        ied = iseed2
        if (iede .lt. 0) iede = iabs(iede)
        if (ied .lt. 0) ied = iabs(ied)
        call rantab_ini(rtab,iseed1,iseed2,iflag)
c
c --- read model data
c ---   'flnm  ' = name of file containing the actual data
c ---   'iexpt ' = experiment number x10  (000=from archive file)
c ---   'yrflag' = days in year flag (0=360J16,1=366J16,2=366J01,3=actual)
c ---   'iyear ' = year of model run
c ---   'jday  ' = julian day of the model run year
c ---   'idm   ' = longitudinal array size
c ---   'jdm   ' = latitudinal  array size
c ---   'kdm   ' = number of layers
        read (*,'(a)') flnm
        write (lp,'(2a)') ' input file: ',trim(flnm)
        call flush(lp)
        call blkini(iexpt, 'iexpt ')
        call blkini(yrflag,'yrflag')
        call blkini(iyear, 'iyear ')
        call blkini(jday,  'jday  ')
        if     (yrflag.eq.3) then
          call jday1900(iyear,jday,jd)
        elseif (yrflag.eq.1 .or. yrflag.eq.2) then
          jd = (iyear-1)*366+jday
        elseif (yrflag.eq.0) then
          jd = (iyear-1)*360+jday
        endif
c
        call blkini(ii,    'idm   ')
        call blkini(jj,    'jdm   ')
        call blkini(kk,    'kdm   ')
        if     (ii.ne.idm .or. jj.ne.jdm) then
          write(lp,*)
          write(lp,*) 'error - wrong idm or jdm (should be:',
     &                                           idm,jdm,')'
          write(lp,*)
          call flush(lp)
          stop
        endif
c
c ---   'thbase' = reference density (sigma units)
        call blkinr(thbase,
     &             'thbase','("blkinr: ",a6," =",f11.4," sig")')
c
c ---   'mthin ' = mask thin layers from output
        call blkinl(mthin, 'mthin ')
c
c ---   'iorign' = i-origin of sampled subregion
c ---   'jorign' = j-origin of sampled subregion
c ---   'idmp  ' = i-extent of sampled subregion (<=idm; 0 implies idm)
c ---   'jdmp  ' = j-extent of sampled subregion (<=jdm; 0 implies jdm)
        call blkini(iorign,'iorign')
        call blkini(jorign,'jorign')
        call blkini(ii,    'idmp  ')
        call blkini(jj,    'jdmp  ')
        if     (ii.eq.0) then
          ii=idm
        endif
        if     (jj.eq.0) then
          jj=jdm
        endif
c
c ---   'iorign,jorign' denote the origin of the subgrid to be extracted 
c ---   from the full history grid (dimensioned idm x jdm). 
c ---   The size of the subgrid is determined by ii,jj.
        write (lp,'(2(a,i5),9x,2(a,i5))') 'extracting i =',iorign,
     &    ' ...',iorign+ii-1,'j =',jorign,' ...',jorign+jj-1
        call flush(lp)
c
c ---   'lonmin' = minimum longitude
c ---   'lonmax' = maximum longitude
c ---   'latmin' = minimum latitude
c ---   'latmax' = maximum latitude
c ---   'depcut' = sst not assimilated shallower than this cutoff depth
c ---   'rnoise' = rms noise level of synthetic sst
c ---   'enoise' = unresolved synoptic eddy noise level
c
        call blkinr(alomin,
     &             'lonmin','("blkinr: ",a6," =",f11.4," sig")')
        call blkinr(alomax,
     &             'lonmax','("blkinr: ",a6," =",f11.4," sig")')
        call blkinr(alamin,
     &             'latmin','("blkinr: ",a6," =",f11.4," sig")')
        call blkinr(alamax,
     &             'latmax','("blkinr: ",a6," =",f11.4," sig")')
        call blkinr(depcut,
     &             'depcut','("blkinr: ",a6," =",f11.4," sig")')
        call blkinr(rnoise,
     &             'rnoise','("blkinr: ",a6," =",f11.4," sig")')
        call blkinr(enoise,
     &             'enoise','("blkinr: ",a6," =",f11.4," sig")')
c
c --- array allocation
c
      call plot_alloc
c
      dpthfil = 'regional.depth'
c
c --- read the archive file.
c
      if (lhycom) then
        call getdat(flnm,time3,artype,initl,icegln,trcout,
     &              iexpt,iversn,yrflag,kkin)       ! hycom input
        time = time3(3)
      else
        call getdtm(flnm,time,initl, thbase)        ! micom input
        artype = 1
        iversn = 10
      endif
c
      write(lp,'(/a,2f8.2/a,2f8.2)') 
     &     'sub-domain longitude range = ',
     &    minval(plon(:,:)),maxval(plon(:,:)),
     &     'sub-domain latitude  range = ',
     &    minval(plat(:,:)),maxval(plat(:,:))
c
      lperiod = ii.eq.idm .and.
     &          maxval(plon(:,:))-minval(plon(:,:)) .gt. 350.0
      if     (lperiod) then
        write(lp,'(/a/)') 'sub-domain assumed to be periodic'
      else
        write(lp,'(/a/)') 'sub-domain assumed to be non-periodic'
      endif
      call flush(lp)
c
      call bigrid(depths)
      call flush(lp)
c
c --- check that bathymetry is consistent with this archive.
c --- only possible with hycom .[ab] file input.
c
      if     (iversn.ge.20) then
        ibadl = 0
        ibads = 0
        do j= 1,jj
          do i= 1,ii
            if     (ip(i,j).eq.1) then
              if     (srfht(i,j).gt.2.0**99) then
                ibads = ibads + 1   ! topo sea, srfht land
*               if     (mod(ibads,100).eq.1) then
*               if     (mod(ibads, 10).eq.1) then
*                 write(lp,*) 'topo sea, srfht land at i,j = ',i,j
*               endif
              endif
            else
              if     (srfht(i,j).lt.2.0**99) then
                ibadl = ibadl + 1   ! topo land, srfht sea
*               if     (mod(ibadl,100).eq.1) then
*               if     (mod(ibadl, 10).eq.1) then
*                 write(lp,*) 'topo land, srfht sea at i,j = ',i,j
*    &                        ,srfht(i,j)
*               endif
              endif
            endif
          enddo
        enddo
        if     (ibads.ne.0) then
          write(lp,*)
          write(lp,*) 'error - wrong bathymetry for this archive file'
          write(lp,*) 'number of topo sea  mismatches = ',ibads
          write(lp,*) 'number of topo land mismatches = ',ibadl
          write(lp,*)
          call flush(lp)
          stop
        endif
        if     (ibadl.ne.0 .and. lhycom) then
          write(lp,*)
*         write(lp,*) 'error - wrong bathymetry for this archive file'
          write(lp,*) 'warning - wrong bathymetry for this archive file'
          write(lp,*) 'number of topo sea  mismatches = ',ibads
          write(lp,*) 'number of topo land mismatches = ',ibadl
          write(lp,*)
          call flush(lp)
*         stop
        endif
      endif !iversn.ge.20
c
      call bigrd1(depths)
c
      do j=1,jj
        do i=1,ii
          p(i,j,1)=0.
          do k=1,kk
            p(i,j,k+1) = p(i,j,k)+dp(i,j,k)
          enddo
        enddo
      enddo
c
c --- -----------------------------------------------------
c --- extract sst data at sampling points and add errors
c --- -----------------------------------------------------
c
c --- sample synthetic sst
c
c
      read(21,*) nptsin
c
      icount=0
      do i=1,nptsin
        read(21,*) alon,alat,depth,dtime,value
        if (alon.gt.180.0) then
          alon2 = alon-360.0
        elseif (alon.lt.-180.0) then
          alon2 = alon+360.0
        else
          alon2=alon
        endif
        alat2 = alat
c
        if(alon2.ge.alomin .and. alon2.le.alomax .and.
     &     alat2.ge.alamin .and. alat2.le.alamax) then
c
          call sample(alon2,alat2,depcut,flag,tmix,iloc,jloc,sst)
c
          if(sst.ne.flag) then
c
            icount = icount+1
c
c --- add random gaussian errors to the synthetic SST
c
            if (rnoise.gt.0.0) then
              call rantab(rtab,iseed1,iseed2,numran)
              sst=sst+rnoise*rtab(numran, 2)
            endif
c
c --- add errors due to unresolved synoptic variability to the synthetic SST
c
            if (enoise.gt.0.0) then
              call rantab(rtab,iseed1,iseed2,numran)
              sst=sst+enoise*rtab(numran, 2)
            endif
c
c --- store observations
c
            alono(icount) = alon
            alato(icount) = alat
            time(icount)  = dtime
            sstao(icount) = sst
c
          endif
c
        endif
c
      enddo
c
c --- write observations to assimilation input file
c
      ndata = icount
      write(22,900) ndata
  900 format(i8)
      do i=1,ndata
        write(22,910) alono(i),alato(i),0.0,time(i),sstao(i)
  910   format(f9.3,f8.3,f5.1,f11.4,f7.3)
      enddo
c
      end
