      program proferr
c
c --- used to add errors to profiles for osses
c
      integer, parameter :: izdm = 901
c
      character flnm*240,frmt*80,cline*80
c
      real dpm,dpms,dsm,dsms,q
      real zdepo(izdm),tempo(izdm),salno(izdm),uo(izdm),vo(izdm),
     &     tho(izdm)
      real twork(izdm),swork(izdm),uwork(izdm),vwork(izdm)
c
c --- convert depth rms error into T and S errors
c
      real*4    rtab(200,2)
      real*4,    allocatable, dimension (:,:,:) ::
     &   tempz,salnz,thz
      real*4    zmax,zdep(izdm)
      integer*4 seed,numran,inisee,iflag,thflag
      integer*2 iseed1,iseed2
      equivalence ( iseed1, iseed2, seed )
c
      real      tenm,onem,temcm,onecm,onemm
      data      tenm/10./,onem/1./,tencm/.1/,onecm/.01/,onemm/.001/
c
c --- errors
c
c     deperr=0.0
c     trnois=0.0
c     tenois=0.0
c     srnois=0.0
c     senois=0.0
c     urnois=0.0
c     uenois=0.0
c     vrnois=0.0
c     venois=0.0
      deperr=2.0
      trnois=0.02
      tenois=0.02 
      srnois=0.05
      senois=0.05 
      urnois=0.02
      uenois=0.02 
      vrnois=0.02
      venois=0.02 
c
c --- initialize random seeds and create random number table
c --- if iflag=1 (iflag=0), then time() is (is not) used to generate seeds
c
      iflag=1 
      call system_clock(inisee)
      seed = 414957000-inisee
      iede = iseed1
      ied = iseed2
      if (iede .lt. 0) iede = iabs(iede)
      if (ied .lt. 0) ied = iabs(ied)
      call rantab_ini(rtab,iseed1,iseed2,iflag)
c
c --- -------------------------------------------------------
c --- extract xbt T profiles and add errors to T, S, u, and v
c --- -------------------------------------------------------
c
c --- sample profiles
c

      open(1,file='temp_model.txt',readonly,status='old')
      open(2,file='temp_model2.txt',form='formatted',status='new')

      kcount=0
      do k=1,izdm
        read(1,*,end=78) dep,tsampl,ssampl,usampl,vsampl,dens
        kcount=kcount+1
        zdepo(k)=dep
        tempo(k)=tsampl
        salno(k)=ssampl
        uo(k)=usampl
        vo(k)=vsampl
        tho(k)=dens
      enddo
   78 continue
      close(1)
c
c --- add random gaussian errors to the variables
c
      do k=1,kcount
        terr=0.0
        serr=0.0
        uerr=0.0
        verr=0.0
        if (trnois.gt.0.0) then
          call rantab(rtab,iseed1,iseed2,numran)
          terr = terr+trnois*rtab(numran,2)
        endif
        if (tenois.gt.0.0) then
          call rantab(rtab,iseed1,iseed2,numran)
          terr = terr+tenois*rtab(numran,2)
        endif
        if (srnois.gt.0.0) then
          call rantab(rtab,iseed1,iseed2,numran)
          serr = serr+srnois*rtab(numran,2)
        endif
        if (senois.gt.0.0) then
          call rantab(rtab,iseed1,iseed2,numran)
          serr = serr+senois*rtab(numran,2)
        endif
        if (urnois.gt.0.0) then
          call rantab(rtab,iseed1,iseed2,numran)
          uerr = uerr+urnois*rtab(numran,2)
        endif
        if (uenois.gt.0.0) then
          call rantab(rtab,iseed1,iseed2,numran)
          uerr = uerr+uenois*rtab(numran,2)
        endif
        if (vrnois.gt.0.0) then
          call rantab(rtab,iseed1,iseed2,numran)
          verr = verr+vrnois*rtab(numran,2)
        endif
        if (venois.gt.0.0) then
          call rantab(rtab,iseed1,iseed2,numran)
          verr = verr+venois*rtab(numran,2)
        endif
c
c --- convert depth rms error into equivalent T error
c --- vertically coherent but tapers to zero toward the surface above 50 m
c
        if (deperr.gt.0.0) then
          if     (k.eq.1) then
            call rantab(rtab,iseed1,iseed2,numran)
            derr = deperr*rtab(numran,2)
            tderr=0.0
            sderr=0.0
            uderr=0.0
            vderr=0.0
          elseif (k.lt.kcount) then
            derr2 = derr*(min(1.0,zdepo(k)/100.0))
            dzdep = zdepo(k-1)-zdepo(k+1)
            dtdz = (tempo(k-1)-tempo(k+1))/dzdep
            tderr = dtdz*derr2
            dsdz = (salno(k-1)-salno(k+1))/dzdep
            sderr = dsdz*derr2
            dudz = (uo(k-1)-uo(k+1))/dzdep
            uderr = dudz*derr2
            dvdz = (vo(k-1)-vo(k+1))/dzdep
            vderr = dvdz*derr2
          endif ! k>=1
        endif ! deperr>0
c
        twork(k)=tempo(k)+terr+tderr
        swork(k)=salno(k)+serr+sderr
        uwork(k)=uo(k)+uerr+sderr
        vwork(k)=vo(k)+verr+sderr
c
      enddo ! k number of depths
c
      do k=1,kcount
        tempo(k)=twork(k)
        salno(k)=swork(k)
        uo(k)=uwork(k)
        vo(k)=vwork(k)
      enddo
c
c --- vertically smooth nsmth times
      nsmth=3
      do ksm=1,nsmth
        do k=2,kcount-1
          twork(k)=0.23*tempo(k-1)+0.54*tempo(k)+0.23*tempo(k+1)
          swork(k)=0.23*salno(k-1)+0.54*salno(k)+0.23*salno(k+1)
          uwork(k)=0.23*uo(k-1)+0.54*uo(k)+0.23*uo(k+1)
          vwork(k)=0.23*vo(k-1)+0.54*vo(k)+0.23*vo(k+1)
        enddo
        do k=2,kcount-1
          tempo(k)=twork(k)
          salno(k)=swork(k)
          uo(k)=uwork(k)
          vo(k)=vwork(k)
        enddo
      enddo
c
c --- write observations with errors
      do k=1,kcount
        write(2,922) zdepo(k),tempo(k),salno(k),uo(k),vo(k),tho(k)
  922   format(f7.1,5f10.5)
      enddo
c
      close(2)
c
      end
      subroutine rantab_ini(rtab,iseed1,iseed2,iflag)
      use mod_plot  ! HYCOM plot array interface
c
      real*4     rtab(200,2)
      integer*2 iseed1,iseed2
      integer*4 iflag
c
c --- get the two seeds
c --- iseed1 is for the 'usual'    random number generator  (udev_rt)
c --- iseed2 is for the 'gaussian' random number generator (grand_rt)
c
      integer*4 seed
      integer*2 isd2(2)
      equivalence (isd2,seed)
c
      call system_clock(inisee)
cdiag write(*,*)'calc itime',time()
      itime = inisee
      seed = 123456789
      iseed1 = isd2(1)
      if (iseed1 .lt. 0) then
        iseed1 = iabs (iseed1)
      endif
      iseed1 = iseed1 + itime*iflag
cdiag write(*,*)'iseed1',iseed1
      seed = 987654321
      iseed2 = isd2(2)
      if (iseed2 .lt. 0) then
        iseed2 = iabs (iseed1)
      endif
      iseed2 = iseed2 + itime*iflag
cdiag write(*,*)'iseed2',iseed2
c
c --- set seeds to negative values if iflag=1
c
      if (iflag .eq. 1) then
        iseed1 = -iseed1
        iseed2 = -iseed2
      endif
c
c --- load the table with a sequence of 200 random numbers.
      do  ic = 1,200
        rtab(ic,1) =  udev_rt(iseed1)
        rtab(ic,2) = grand_rt(iseed2)
      enddo
cdiag write(*,*)'rtabs',rtab(100,1),rtab(100,2)
c
      return
      end subroutine rantab_ini

      subroutine rantab (rtab,iseed1,iseed2,numran)
c
c --- each time this routine is called one of the 200 records in "rtab"
c --- is randomly selected and new random numbers are inserted
c
      real*4 rtab(200,2)
c
c --- pick a number between 1 and 200
      numran = nint(1+199*udev_rt(iseed1))
      if (numran .gt. 200) then
        numran = 200
      endif
c
c --- get new random numbers
      rtab(numran,1) =  udev_rt(iseed1)
      rtab(numran,2) = grand_rt(iseed2)
c
      return
      end subroutine rantab
c
      real function grand_rt(idum)
c
c provides random number from gaussian distribution, mean 0, varaince 1,
c for finding random component of turbulent u
c
c from arthur mariano using routines from numerical recipes
c
      data iset/0/
      save iset,gset
c
      if (iset.eq.0) then
 1      v1=2.*udev_rt(idum)-1.0
        v2=2.*udev_rt(idum)-1.0
        r=v1**2+v2**2
        if (r.ge.1.0)go to 1
        fac=sqrt(-2.*log(r)/r)
        gset=v1*fac
        grand_rt=v2*fac
        iset=1
      else
        grand_rt=gset
        iset=0
      endif
c
      return
      end function grand_rt
c
      real function udev_rt(iseed)

      parameter (m=714025,ia=1366,ic=150889,rm=1./m)

      dimension ir(97)

      data iff / 0 /
c
      if (iseed .lt. 0 .or. iff .eq. 0) then
        iff = 1
        iseed = mod(ic-iseed,m)
        do j=1,97
          iseed = mod(ia*iseed+ic,m)
          ir(j) = iseed
        enddo
        iseed = mod(ia*iseed+ic,m)
      else
        iseed = mod(iseed,m)
      endif
      iy=iseed
      j = min(97,1+(97*iy)/m)
      iy = ir(j)
      udev_rt = iy*rm
      iseed = mod(ia*iseed+ic,m)
      ir(j) = iseed
      return
      end function udev_rt
