      subroutine restep(temold,salold,dnsold,pold,
     .                  temnew,salnew,dnsnew,pnew,theta,kold,knew)
c
c --- convert stairstep (i.e., piecewise constant) T/S/dens profiles
c --- into stairstep profiles constrained to have prescribed density ('theta')
c --- steps.
c
c --- input  variables: temold,salold,dnsold,pold,kold,theta
c --- output variables: temnew,salnew,dnsnew,pnew,knew
c
      implicit none
c
      integer kold,knew,ko,kmx
c
      real temold(kold),salold(kold),dnsold(kold),pold(kold+1)
c
      real temnew(knew),salnew(knew),dnsnew(knew),pnew(knew+1),
     &     theta(knew)
c
      parameter (kmx=99)
      real cloutt,clouts,colint,colins,
     .     pinteg,uinteg,vinteg,tinteg,sinteg,phi,plo,pa,pb,siga,
     .     sigb,q,acurcy,oldsig(kmx)
      logical at_top,abort
      data acurcy/1.e-5/
      data abort/.false./
c
      if (max(kold,knew).gt.kmx) then
        write (lp,'(a,i4)') 'restep error -- increase kmx to',
     .    max(kold,knew)
        abort=.true.
      end if
c
c$OMP PARALLEL DO PRIVATE(colint,colins,colinr,cloutt,clouts,
c$OMP+ cloutr,pinteg,tinteg,sinteg,siga,sigb,at_top,
c$OMP+ phi,plo,pa,pb,q,oldsig)
c
      do k=1,knew
        dnsnew(k)=theta(k)
      enddo
c
c --- remove density inversions from input profile
      oldsig(1)=dnsold(1)
      do k=2,kold
        oldsig(k)=max(oldsig(k-1),dnsold(k))
      enddo
c
      dnsnew(   1)=min(oldsig(1),oldsig(kold),theta(   1))
      dnsnew(knew)=max(oldsig(1),oldsig(kold),theta(knew))
      pnew(     1)=pold(     1)
      pnew(knew+1)=pold(kold+1)
c
c --- column integrals (colin/clout) are computed for diagnostic purposes only
      cloutr=0.
      colinr=oldsig(kold)*(pold(kold+1)-pold(kold))
      colint=temold(kold)*(pold(kold+1)-pold(kold))
      colins=salold(kold)*(pold(kold+1)-pold(kold))
c
      do k=1,kold-1
        colinr=colinr+oldsig(k)*(pold(k+1)-pold(k))
        colint=colint+temold(k)*(pold(k+1)-pold(k))
        colins=colins+salold(k)*(pold(k+1)-pold(k))
      enddo
c
c --- find interface depth pnew(k+1) separating layers k and k+1 by requiring 
c --- that integral over p*d(sigma) from dnsnew(k) to dnsnew(k+1) be preserved.
c
      do 4 k=1,knew-1
      pinteg=0.
      sigb=dnsnew(k)
      do ko=1,kold
        siga=sigb
        sigb=min(dnsnew(k+1),max(dnsnew(k),oldsig(ko)))
        pinteg=pinteg+pold(ko)*(sigb-siga)
        if (oldsig(ko).ge.dnsnew(k+1)) go to 25
      enddo
      siga=sigb
      sigb=dnsnew(k+1)
      pinteg=pinteg+pold(kold+1)*(sigb-siga)
c
 25   pnew(k+1)=pinteg/(dnsnew(k+1)-dnsnew(k))
      cloutr=cloutr+dnsnew(k)*(pnew(k+1)-pnew(k))
c --- remove effect of roundoff errors on monotonicity
ccc   pnew(k+1)=max(pnew(k),min(pnew(k+1),pnew(knew+1)))
 4    continue
c
      cloutr=cloutr+dnsnew(knew)*
     .   (pnew(knew+1)-pnew(knew))
      if (abs(cloutr-colinr).gt.acurcy*abs(colinr)) then
     .  write (lp,100) ' restep - dens.column intgl.error',
     .   colinr,cloutr,(cloutr-colinr)/colinr
      endif
c
c --- ---------------------------------------
c --- integrate -t,s over new depth intervals
c --- ---------------------------------------
c
      at_top=.true.
      clouts=0.
      cloutt=0.
c
      do 8 k=1,knew
      tinteg=0.
      sinteg=0.
      phi=pnew(k+1)
      plo=pnew(k  )
      pb=plo
      do ko=1,kold
        pa=pb
        pb=min(phi,max(plo,pold(ko+1)))
        tinteg=tinteg+temold(ko)*(pb-pa)
        sinteg=sinteg+salold(ko)*(pb-pa)
        if (pa.ge.phi) go to 28
      enddo
      pa=pb
      pb=phi
      tinteg=tinteg+temold(kold)*(pb-pa)
      sinteg=sinteg+salold(kold)*(pb-pa)
 28   cloutt=cloutt+tinteg
      clouts=clouts+sinteg
c
      q=phi-plo
      if (q.gt.0.) then
        temnew(k)=tinteg/q
        salnew(k)=sinteg/q
        at_top=.false.
      else if (at_top) then
        temnew(k)=temold(1)
        salnew(k)=salold(1)
      else
        temnew(k)=temold(kold)
        salnew(k)=salold(kold)
      end if
 8    continue
c
      if (abs(cloutt-colint).gt.acurcy*5.*pold(kold+1))
     .  write (lp,100) ' restep - t column intgl.error',
     .   colint,cloutt,(cloutt-colint)/colint
      if (abs(clouts-colins).gt.acurcy*5.*pold(kold+1))
     .  write (lp,100) ' restep - s column intgl.error',
     .   colins,clouts,(clouts-colins)/colins
 100  format (a,1p,2e14.6,e9.1)
c
c$OMP END PARALLEL DO
c
      if (abort) stop '(reflux error)'
c
      return
      end
